import { debounce, throttle } from "lodash-es";
import factory from "codemirror-ssr";
import usePlaceholder from "codemirror-ssr/addon/display/placeholder.js";
import useOverlay from "codemirror-ssr/addon/mode/overlay.js";
import useXml from "codemirror-ssr/mode/xml/xml.js";
import useMarkdown from "codemirror-ssr/mode/markdown/markdown.js";
import useGfm from "codemirror-ssr/mode/gfm/gfm.js";
import useYaml from "codemirror-ssr/mode/yaml/yaml.js";
import useYamlFrontmatter from "codemirror-ssr/mode/yaml-frontmatter/yaml-frontmatter.js";
import useContinuelist from "codemirror-ssr/addon/edit/continuelist.js";
import selectFiles from "select-files";
import * as icons from "@icon-park/svg";
import { delegate } from "tippy.js";
import { unified } from "unified";
import remarkParse from "remark-parse";
import remarkRehype from "remark-rehype";
import rehypeRaw from "rehype-raw";
import rehypeSanitize from "rehype-sanitize";
import rehypeStringify from "rehype-stringify";
import { defaultSchema } from "hast-util-sanitize";
import { visit } from "unist-util-visit";
import wordCount from "word-count";
var index = /* @__PURE__ */ (() => `/* BASICS */

.CodeMirror {
  /* Set height, width, borders, and global font properties here */
  font-family: monospace;
  height: 300px;
  color: black;
  direction: ltr;
}

/* PADDING */

.CodeMirror-lines {
  padding: 4px 0; /* Vertical padding around content */
}

.CodeMirror pre.CodeMirror-line,
.CodeMirror pre.CodeMirror-line-like {
  padding: 0 4px; /* Horizontal padding of content */
}

.CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {
  background-color: white; /* The little square between H and V scrollbars */
}

/* GUTTER */

.CodeMirror-gutters {
  border-right: 1px solid #ddd;
  background-color: #f7f7f7;
  white-space: nowrap;
}

.CodeMirror-linenumbers {}

.CodeMirror-linenumber {
  padding: 0 3px 0 5px;
  min-width: 20px;
  text-align: right;
  color: #999;
  white-space: nowrap;
}

.CodeMirror-guttermarker { color: black; }

.CodeMirror-guttermarker-subtle { color: #999; }

/* CURSOR */

.CodeMirror-cursor {
  border-left: 1px solid black;
  border-right: none;
  width: 0;
}

/* Shown when moving in bi-directional text */

.CodeMirror div.CodeMirror-secondarycursor {
  border-left: 1px solid silver;
}

.cm-fat-cursor .CodeMirror-cursor {
  width: auto;
  border: 0 !important;
  background: #7e7;
}

.cm-fat-cursor div.CodeMirror-cursors {
  z-index: 1;
}

.cm-fat-cursor .CodeMirror-line::selection,
.cm-fat-cursor .CodeMirror-line > span::selection, 
.cm-fat-cursor .CodeMirror-line > span > span::selection { background: transparent; }

.cm-fat-cursor .CodeMirror-line::-moz-selection,
.cm-fat-cursor .CodeMirror-line > span::-moz-selection,
.cm-fat-cursor .CodeMirror-line > span > span::-moz-selection { background: transparent; }

.cm-fat-cursor { caret-color: transparent; }

@-moz-keyframes blink {
  0% {}
  50% { background-color: transparent; }
  100% {}
}

@-webkit-keyframes blink {
  0% {}
  50% { background-color: transparent; }
  100% {}
}

@keyframes blink {
  0% {}
  50% { background-color: transparent; }
  100% {}
}

/* Can style cursor different in overwrite (non-insert) mode */

.CodeMirror-overwrite .CodeMirror-cursor {}

.cm-tab { display: inline-block; text-decoration: inherit; }

.CodeMirror-rulers {
  position: absolute;
  left: 0; right: 0; top: -50px; bottom: 0;
  overflow: hidden;
}

.CodeMirror-ruler {
  border-left: 1px solid #ccc;
  top: 0; bottom: 0;
  position: absolute;
}

/* DEFAULT THEME */

.cm-s-default .cm-header {color: blue;}

.cm-s-default .cm-quote {color: #090;}

.cm-negative {color: #d44;}

.cm-positive {color: #292;}

.cm-header, .cm-strong {font-weight: bold;}

.cm-em {font-style: italic;}

.cm-link {text-decoration: underline;}

.cm-strikethrough {text-decoration: line-through;}

.cm-s-default .cm-keyword {color: #708;}

.cm-s-default .cm-atom {color: #219;}

.cm-s-default .cm-number {color: #164;}

.cm-s-default .cm-def {color: #00f;}

.cm-s-default .cm-variable,
.cm-s-default .cm-punctuation,
.cm-s-default .cm-property,
.cm-s-default .cm-operator {}

.cm-s-default .cm-variable-2 {color: #05a;}

.cm-s-default .cm-variable-3, .cm-s-default .cm-type {color: #085;}

.cm-s-default .cm-comment {color: #a50;}

.cm-s-default .cm-string {color: #a11;}

.cm-s-default .cm-string-2 {color: #f50;}

.cm-s-default .cm-meta {color: #555;}

.cm-s-default .cm-qualifier {color: #555;}

.cm-s-default .cm-builtin {color: #30a;}

.cm-s-default .cm-bracket {color: #997;}

.cm-s-default .cm-tag {color: #170;}

.cm-s-default .cm-attribute {color: #00c;}

.cm-s-default .cm-hr {color: #999;}

.cm-s-default .cm-link {color: #00c;}

.cm-s-default .cm-error {color: #f00;}

.cm-invalidchar {color: #f00;}

.CodeMirror-composing { border-bottom: 2px solid; }

/* Default styles for common addons */

div.CodeMirror span.CodeMirror-matchingbracket {color: #0b0;}

div.CodeMirror span.CodeMirror-nonmatchingbracket {color: #a22;}

.CodeMirror-matchingtag { background: rgba(255, 150, 0, .3); }

.CodeMirror-activeline-background {background: #e8f2ff;}

/* STOP */

/* The rest of this file contains styles related to the mechanics of
   the editor. You probably shouldn't touch them. */

.CodeMirror {
  position: relative;
  overflow: hidden;
  background: white;
}

.CodeMirror-scroll {
  overflow: scroll !important; /* Things will break if this is overridden */
  /* 50px is the magic margin used to hide the element's real scrollbars */
  /* See overflow: hidden in .CodeMirror */
  margin-bottom: -50px; margin-right: -50px;
  padding-bottom: 50px;
  height: 100%;
  outline: none; /* Prevent dragging from highlighting the element */
  position: relative;
  z-index: 0;
}

.CodeMirror-sizer {
  position: relative;
  border-right: 50px solid transparent;
}

/* The fake, visible scrollbars. Used to force redraw during scrolling
   before actual scrolling happens, thus preventing shaking and
   flickering artifacts. */

.CodeMirror-vscrollbar, .CodeMirror-hscrollbar, .CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {
  position: absolute;
  z-index: 6;
  display: none;
  outline: none;
}

.CodeMirror-vscrollbar {
  right: 0; top: 0;
  overflow-x: hidden;
  overflow-y: scroll;
}

.CodeMirror-hscrollbar {
  bottom: 0; left: 0;
  overflow-y: hidden;
  overflow-x: scroll;
}

.CodeMirror-scrollbar-filler {
  right: 0; bottom: 0;
}

.CodeMirror-gutter-filler {
  left: 0; bottom: 0;
}

.CodeMirror-gutters {
  position: absolute; left: 0; top: 0;
  min-height: 100%;
  z-index: 3;
}

.CodeMirror-gutter {
  white-space: normal;
  height: 100%;
  display: inline-block;
  vertical-align: top;
  margin-bottom: -50px;
}

.CodeMirror-gutter-wrapper {
  position: absolute;
  z-index: 4;
  background: none !important;
  border: none !important;
}

.CodeMirror-gutter-background {
  position: absolute;
  top: 0; bottom: 0;
  z-index: 4;
}

.CodeMirror-gutter-elt {
  position: absolute;
  cursor: default;
  z-index: 4;
}

.CodeMirror-gutter-wrapper ::selection { background-color: transparent }

.CodeMirror-gutter-wrapper ::-moz-selection { background-color: transparent }

.CodeMirror-lines {
  cursor: text;
  min-height: 1px; /* prevents collapsing before first draw */
}

.CodeMirror pre.CodeMirror-line,
.CodeMirror pre.CodeMirror-line-like {
  /* Reset some styles that the rest of the page might have set */
  -moz-border-radius: 0; -webkit-border-radius: 0; border-radius: 0;
  border-width: 0;
  background: transparent;
  font-family: inherit;
  font-size: inherit;
  margin: 0;
  white-space: pre;
  word-wrap: normal;
  line-height: inherit;
  color: inherit;
  z-index: 2;
  position: relative;
  overflow: visible;
  -webkit-tap-highlight-color: transparent;
  -webkit-font-variant-ligatures: contextual;
  font-variant-ligatures: contextual;
}

.CodeMirror-wrap pre.CodeMirror-line,
.CodeMirror-wrap pre.CodeMirror-line-like {
  word-wrap: break-word;
  white-space: pre-wrap;
  word-break: normal;
}

.CodeMirror-linebackground {
  position: absolute;
  left: 0; right: 0; top: 0; bottom: 0;
  z-index: 0;
}

.CodeMirror-linewidget {
  position: relative;
  z-index: 2;
  padding: 0.1px; /* Force widget margins to stay inside of the container */
}

.CodeMirror-widget {}

.CodeMirror-rtl pre { direction: rtl; }

.CodeMirror-code {
  outline: none;
}

/* Force content-box sizing for the elements where we expect it */

.CodeMirror-scroll,
.CodeMirror-sizer,
.CodeMirror-gutter,
.CodeMirror-gutters,
.CodeMirror-linenumber {
  -moz-box-sizing: content-box;
  box-sizing: content-box;
}

.CodeMirror-measure {
  position: absolute;
  width: 100%;
  height: 0;
  overflow: hidden;
  visibility: hidden;
}

.CodeMirror-cursor {
  position: absolute;
  pointer-events: none;
}

.CodeMirror-measure pre { position: static; }

div.CodeMirror-cursors {
  visibility: hidden;
  position: relative;
  z-index: 3;
}

div.CodeMirror-dragcursors {
  visibility: visible;
}

.CodeMirror-focused div.CodeMirror-cursors {
  visibility: visible;
}

.CodeMirror-selected { background: #d9d9d9; }

.CodeMirror-focused .CodeMirror-selected { background: #d7d4f0; }

.CodeMirror-crosshair { cursor: crosshair; }

.CodeMirror-line::selection, .CodeMirror-line > span::selection, .CodeMirror-line > span > span::selection { background: #d7d4f0; }

.CodeMirror-line::-moz-selection, .CodeMirror-line > span::-moz-selection, .CodeMirror-line > span > span::-moz-selection { background: #d7d4f0; }

.cm-searching {
  background-color: #ffa;
  background-color: rgba(255, 255, 0, .4);
}

/* Used to force a border model for a node */

.cm-force-border { padding-right: .1px; }

@media print {
  /* Hide the cursor when printing */
  .CodeMirror div.CodeMirror-cursors {
    visibility: hidden;
  }
}

/* See issue #2901 */

.cm-tab-wrap-hack:after { content: ''; }

/* Help users use markselection to safely style text background */

span.CodeMirror-selectedtext { background: none; }

.tippy-box[data-animation=fade][data-state=hidden]{opacity:0}

[data-tippy-root]{max-width:calc(100vw - 10px)}

.tippy-box{position:relative;background-color:#333;color:#fff;border-radius:4px;font-size:14px;line-height:1.4;white-space:normal;outline:0;transition-property:transform,visibility,opacity}

.tippy-box[data-placement^=top]>.tippy-arrow{bottom:0}

.tippy-box[data-placement^=top]>.tippy-arrow:before{bottom:-7px;left:0;border-width:8px 8px 0;border-top-color:initial;transform-origin:center top}

.tippy-box[data-placement^=bottom]>.tippy-arrow{top:0}

.tippy-box[data-placement^=bottom]>.tippy-arrow:before{top:-7px;left:0;border-width:0 8px 8px;border-bottom-color:initial;transform-origin:center bottom}

.tippy-box[data-placement^=left]>.tippy-arrow{right:0}

.tippy-box[data-placement^=left]>.tippy-arrow:before{border-width:8px 0 8px 8px;border-left-color:initial;right:-7px;transform-origin:center left}

.tippy-box[data-placement^=right]>.tippy-arrow{left:0}

.tippy-box[data-placement^=right]>.tippy-arrow:before{left:-7px;border-width:8px 8px 8px 0;border-right-color:initial;transform-origin:center right}

.tippy-box[data-inertia][data-state=visible]{transition-timing-function:cubic-bezier(.54,1.5,.38,1.11)}

.tippy-arrow{width:16px;height:16px;color:#333}

.tippy-arrow:before{content:"";position:absolute;border-color:transparent;border-style:solid}

.tippy-content{position:relative;padding:5px 9px;z-index:1}

.tippy-box[data-theme~=light-border]{background-color:#fff;background-clip:padding-box;border:1px solid rgba(0,8,16,.15);color:#333;box-shadow:0 4px 14px -2px rgba(0,8,16,.08)}

.tippy-box[data-theme~=light-border]>.tippy-backdrop{background-color:#fff}

.tippy-box[data-theme~=light-border]>.tippy-arrow:after,.tippy-box[data-theme~=light-border]>.tippy-svg-arrow:after{content:"";position:absolute;z-index:-1}

.tippy-box[data-theme~=light-border]>.tippy-arrow:after{border-color:transparent;border-style:solid}

.tippy-box[data-theme~=light-border][data-placement^=top]>.tippy-arrow:before{border-top-color:#fff}

.tippy-box[data-theme~=light-border][data-placement^=top]>.tippy-arrow:after{border-top-color:rgba(0,8,16,.2);border-width:7px 7px 0;top:17px;left:1px}

.tippy-box[data-theme~=light-border][data-placement^=top]>.tippy-svg-arrow>svg{top:16px}

.tippy-box[data-theme~=light-border][data-placement^=top]>.tippy-svg-arrow:after{top:17px}

.tippy-box[data-theme~=light-border][data-placement^=bottom]>.tippy-arrow:before{border-bottom-color:#fff;bottom:16px}

.tippy-box[data-theme~=light-border][data-placement^=bottom]>.tippy-arrow:after{border-bottom-color:rgba(0,8,16,.2);border-width:0 7px 7px;bottom:17px;left:1px}

.tippy-box[data-theme~=light-border][data-placement^=bottom]>.tippy-svg-arrow>svg{bottom:16px}

.tippy-box[data-theme~=light-border][data-placement^=bottom]>.tippy-svg-arrow:after{bottom:17px}

.tippy-box[data-theme~=light-border][data-placement^=left]>.tippy-arrow:before{border-left-color:#fff}

.tippy-box[data-theme~=light-border][data-placement^=left]>.tippy-arrow:after{border-left-color:rgba(0,8,16,.2);border-width:7px 0 7px 7px;left:17px;top:1px}

.tippy-box[data-theme~=light-border][data-placement^=left]>.tippy-svg-arrow>svg{left:11px}

.tippy-box[data-theme~=light-border][data-placement^=left]>.tippy-svg-arrow:after{left:12px}

.tippy-box[data-theme~=light-border][data-placement^=right]>.tippy-arrow:before{border-right-color:#fff;right:16px}

.tippy-box[data-theme~=light-border][data-placement^=right]>.tippy-arrow:after{border-width:7px 7px 7px 0;right:17px;top:1px;border-right-color:rgba(0,8,16,.2)}

.tippy-box[data-theme~=light-border][data-placement^=right]>.tippy-svg-arrow>svg{right:11px}

.tippy-box[data-theme~=light-border][data-placement^=right]>.tippy-svg-arrow:after{right:12px}

.tippy-box[data-theme~=light-border]>.tippy-svg-arrow{fill:#fff}

.tippy-box[data-theme~=light-border]>.tippy-svg-arrow:after{background-image:url(data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMTYiIGhlaWdodD0iNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj48cGF0aCBkPSJNMCA2czEuNzk2LS4wMTMgNC42Ny0zLjYxNUM1Ljg1MS45IDYuOTMuMDA2IDggMGMxLjA3LS4wMDYgMi4xNDguODg3IDMuMzQzIDIuMzg1QzE0LjIzMyA2LjAwNSAxNiA2IDE2IDZIMHoiIGZpbGw9InJnYmEoMCwgOCwgMTYsIDAuMikiLz48L3N2Zz4=);background-size:16px 6px;width:16px;height:6px}

.bytemd {
  box-sizing: border-box;
  /* Force content-box sizing for the elements where we expect it */
  font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Helvetica, Arial, sans-serif, Apple Color Emoji, Segoe UI Emoji;
  color: #24292e;
  border: 1px solid #e1e4e8;
  background-color: #fff;
  height: 300px;
}

.bytemd * {
  box-sizing: border-box;
}

.bytemd-hidden {
  display: none !important;
}

.bytemd .CodeMirror-scroll,
.bytemd .CodeMirror-sizer,
.bytemd .CodeMirror-gutter,
.bytemd .CodeMirror-gutters,
.bytemd .CodeMirror-linenumber {
  box-sizing: content-box;
}

.bytemd .CodeMirror,
.bytemd code,
.bytemd kbd {
  font-family: SFMono-Regular, Consolas, Liberation Mono, Menlo, monospace;
}

.bytemd-toolbar {
  padding: 4px 12px;
  border-bottom: 1px solid #e1e4e8;
  background-color: #fafbfc;
  user-select: none;
  overflow: hidden;
}

.bytemd-toolbar-left {
  float: left;
}

.bytemd-toolbar-right {
  float: right;
}

.bytemd-toolbar-tab {
  display: inline-block;
  cursor: pointer;
  padding-left: 8px;
  padding-right: 8px;
  line-height: 24px;
  font-size: 14px;
}

.bytemd-toolbar-tab-active {
  color: #0366d6;
}

.bytemd-toolbar-icon {
  display: inline-block;
  vertical-align: top;
  cursor: pointer;
  border-radius: 4px;
  margin-left: 6px;
  margin-right: 6px;
}

.bytemd-toolbar-icon svg,
.bytemd-toolbar-icon img {
  display: block;
  padding: 4px;
  width: 24px;
  height: 24px;
}

.bytemd-toolbar-icon:hover {
  background-color: #e1e4e8;
}

.bytemd-toolbar-icon-active {
  color: #0366d6;
}

.bytemd-toolbar .tippy-content {
  padding-left: 0;
  padding-right: 0;
}

.bytemd-dropdown {
  max-height: 300px;
  overflow: auto;
  font-size: 14px;
}

.bytemd-dropdown-title {
  margin: 0 12px;
  font-weight: 500;
  border-bottom: 1px solid #e1e4e8;
  line-height: 32px;
  color: #444d56;
}

.bytemd-dropdown-item {
  padding: 4px 12px;
  height: 32px;
  cursor: pointer;
}

.bytemd-dropdown-item:hover {
  background-color: #f6f8fa;
}

.bytemd-dropdown-item-icon {
  display: inline-block;
}

.bytemd-dropdown-item-icon svg {
  display: block;
  padding: 4px;
  width: 24px;
  height: 24px;
}

.bytemd-dropdown-item-title {
  display: inline-block;
  line-height: 24px;
  vertical-align: top;
}

.bytemd-body {
  height: calc(100% - 33px - 25px);
  overflow: auto;
}

.bytemd-editor {
  display: inline-block;
  vertical-align: top;
  height: 100%;
  overflow: hidden;
}

.bytemd-editor .CodeMirror {
  height: 100%;
  font-size: 14px;
  line-height: 1.5;
}

.bytemd-editor .CodeMirror pre.CodeMirror-placeholder {
  color: #959da5;
}

.bytemd-editor .CodeMirror .CodeMirror-lines {
  max-width: 800px;
  margin: 0 auto;
  padding: 16px 0;
}

.bytemd-editor .CodeMirror pre.CodeMirror-line,
.bytemd-editor .CodeMirror pre.CodeMirror-line-like {
  padding: 0 4%;
}

.bytemd-preview {
  display: inline-block;
  vertical-align: top;
  height: 100%;
  overflow: auto;
}

.bytemd-preview .markdown-body {
  max-width: 800px;
  margin: 0 auto;
  padding: 16px 4%;
}

.bytemd-sidebar {
  display: inline-block;
  vertical-align: top;
  height: 100%;
  overflow: auto;
  font-size: 16px;
  border-left: 1px solid #e1e4e8;
  width: 280px;
  position: relative;
  padding: 0 16px;
}

.bytemd-sidebar-close {
  position: absolute;
  padding: 16px;
  top: 0;
  right: 0;
  cursor: pointer;
}

.bytemd-sidebar-close:hover {
  color: #0366d6;
}

.bytemd-sidebar h2 {
  font-size: 16px;
  font-weight: 600;
  margin: 32px 0 16px;
}

.bytemd-sidebar ul {
  padding-left: 0;
  color: #959da5;
}

.bytemd-help {
  font-size: 13px;
}

.bytemd-help ul {
  line-height: 20px;
}

.bytemd-help ul svg {
  width: 16px;
  height: 16px;
  display: block;
}

.bytemd-help ul div {
  display: inline-block;
  vertical-align: top;
}

.bytemd-help li {
  list-style: none;
  margin-bottom: 12px;
}

.bytemd-help-icon {
  padding: 2px 0;
}

.bytemd-help-title {
  padding-left: 8px;
}

.bytemd-help-content {
  float: right;
  font-size: 12px;
}

.bytemd-toc li {
  list-style: none;
  margin-bottom: 4px;
  font-size: 14px;
  line-height: 2;
  cursor: pointer;
  text-overflow: ellipsis;
  overflow: hidden;
  white-space: nowrap;
}

.bytemd-toc-active {
  color: #0366d6;
  background-color: #f6f8fa;
}

.bytemd-toc-first {
  font-weight: 500;
}

.bytemd-status {
  font-size: 12px;
  line-height: 24px;
  border-top: 1px solid #e1e4e8;
  user-select: none;
}

.bytemd-status-left {
  float: left;
}

.bytemd-status-left span {
  padding-left: 16px;
}

.bytemd-status-left strong {
  font-weight: 600;
}

.bytemd-status-right {
  float: right;
}

.bytemd-status-right label,
.bytemd-status-right span {
  margin-right: 16px;
  cursor: pointer;
}

.bytemd-status-right span:hover {
  color: #0366d6;
}

.bytemd-status-right input {
  vertical-align: middle;
  margin-right: 3px;
}

.bytemd-status-error {
  color: #d73a49;
}

.bytemd-fullscreen.bytemd {
  position: fixed;
  left: 0;
  right: 0;
  top: 0;
  bottom: 0;
  border: none;
  height: 100vh !important;
}

.bytemd-split .bytemd-preview {
  border-left: 1px solid #e1e4e8;
}

.tippy-box {
  font-size: 12px;
}`)();
function noop() {
}
function run(fn) {
  return fn();
}
function blank_object() {
  return /* @__PURE__ */ Object.create(null);
}
function run_all(fns) {
  fns.forEach(run);
}
function is_function(thing) {
  return typeof thing === "function";
}
function safe_not_equal(a, b) {
  return a != a ? b == b : a !== b || (a && typeof a === "object" || typeof a === "function");
}
function not_equal(a, b) {
  return a != a ? b == b : a !== b;
}
function is_empty(obj) {
  return Object.keys(obj).length === 0;
}
function append(target, node) {
  target.appendChild(node);
}
function insert(target, node, anchor) {
  target.insertBefore(node, anchor || null);
}
function detach(node) {
  node.parentNode.removeChild(node);
}
function destroy_each(iterations, detaching) {
  for (let i = 0; i < iterations.length; i += 1) {
    if (iterations[i])
      iterations[i].d(detaching);
  }
}
function element(name) {
  return document.createElement(name);
}
function text(data) {
  return document.createTextNode(data);
}
function empty() {
  return text("");
}
function listen(node, event, handler, options) {
  node.addEventListener(event, handler, options);
  return () => node.removeEventListener(event, handler, options);
}
function attr(node, attribute, value) {
  if (value == null)
    node.removeAttribute(attribute);
  else if (node.getAttribute(attribute) !== value)
    node.setAttribute(attribute, value);
}
function children(element2) {
  return Array.from(element2.childNodes);
}
function set_data(text2, data) {
  data = "" + data;
  if (text2.wholeText !== data)
    text2.data = data;
}
function toggle_class(element2, name, toggle) {
  element2.classList[toggle ? "add" : "remove"](name);
}
function custom_event(type, detail, { bubbles = false, cancelable = false } = {}) {
  const e = document.createEvent("CustomEvent");
  e.initCustomEvent(type, bubbles, cancelable, detail);
  return e;
}
let current_component;
function set_current_component(component) {
  current_component = component;
}
function get_current_component() {
  if (!current_component)
    throw new Error("Function called outside component initialization");
  return current_component;
}
function onMount(fn) {
  get_current_component().$$.on_mount.push(fn);
}
function afterUpdate(fn) {
  get_current_component().$$.after_update.push(fn);
}
function onDestroy(fn) {
  get_current_component().$$.on_destroy.push(fn);
}
function createEventDispatcher() {
  const component = get_current_component();
  return (type, detail, { cancelable = false } = {}) => {
    const callbacks = component.$$.callbacks[type];
    if (callbacks) {
      const event = custom_event(type, detail, { cancelable });
      callbacks.slice().forEach((fn) => {
        fn.call(component, event);
      });
      return !event.defaultPrevented;
    }
    return true;
  };
}
const dirty_components = [];
const binding_callbacks = [];
const render_callbacks = [];
const flush_callbacks = [];
const resolved_promise = Promise.resolve();
let update_scheduled = false;
function schedule_update() {
  if (!update_scheduled) {
    update_scheduled = true;
    resolved_promise.then(flush);
  }
}
function tick() {
  schedule_update();
  return resolved_promise;
}
function add_render_callback(fn) {
  render_callbacks.push(fn);
}
const seen_callbacks = /* @__PURE__ */ new Set();
let flushidx = 0;
function flush() {
  const saved_component = current_component;
  do {
    while (flushidx < dirty_components.length) {
      const component = dirty_components[flushidx];
      flushidx++;
      set_current_component(component);
      update(component.$$);
    }
    set_current_component(null);
    dirty_components.length = 0;
    flushidx = 0;
    while (binding_callbacks.length)
      binding_callbacks.pop()();
    for (let i = 0; i < render_callbacks.length; i += 1) {
      const callback = render_callbacks[i];
      if (!seen_callbacks.has(callback)) {
        seen_callbacks.add(callback);
        callback();
      }
    }
    render_callbacks.length = 0;
  } while (dirty_components.length);
  while (flush_callbacks.length) {
    flush_callbacks.pop()();
  }
  update_scheduled = false;
  seen_callbacks.clear();
  set_current_component(saved_component);
}
function update($$) {
  if ($$.fragment !== null) {
    $$.update();
    run_all($$.before_update);
    const dirty = $$.dirty;
    $$.dirty = [-1];
    $$.fragment && $$.fragment.p($$.ctx, dirty);
    $$.after_update.forEach(add_render_callback);
  }
}
const outroing = /* @__PURE__ */ new Set();
let outros;
function group_outros() {
  outros = {
    r: 0,
    c: [],
    p: outros
  };
}
function check_outros() {
  if (!outros.r) {
    run_all(outros.c);
  }
  outros = outros.p;
}
function transition_in(block, local) {
  if (block && block.i) {
    outroing.delete(block);
    block.i(local);
  }
}
function transition_out(block, local, detach2, callback) {
  if (block && block.o) {
    if (outroing.has(block))
      return;
    outroing.add(block);
    outros.c.push(() => {
      outroing.delete(block);
      if (callback) {
        if (detach2)
          block.d(1);
        callback();
      }
    });
    block.o(local);
  } else if (callback) {
    callback();
  }
}
function create_component(block) {
  block && block.c();
}
function mount_component(component, target, anchor, customElement) {
  const { fragment, on_mount, on_destroy, after_update } = component.$$;
  fragment && fragment.m(target, anchor);
  if (!customElement) {
    add_render_callback(() => {
      const new_on_destroy = on_mount.map(run).filter(is_function);
      if (on_destroy) {
        on_destroy.push(...new_on_destroy);
      } else {
        run_all(new_on_destroy);
      }
      component.$$.on_mount = [];
    });
  }
  after_update.forEach(add_render_callback);
}
function destroy_component(component, detaching) {
  const $$ = component.$$;
  if ($$.fragment !== null) {
    run_all($$.on_destroy);
    $$.fragment && $$.fragment.d(detaching);
    $$.on_destroy = $$.fragment = null;
    $$.ctx = [];
  }
}
function make_dirty(component, i) {
  if (component.$$.dirty[0] === -1) {
    dirty_components.push(component);
    schedule_update();
    component.$$.dirty.fill(0);
  }
  component.$$.dirty[i / 31 | 0] |= 1 << i % 31;
}
function init(component, options, instance2, create_fragment2, not_equal2, props, append_styles, dirty = [-1]) {
  const parent_component = current_component;
  set_current_component(component);
  const $$ = component.$$ = {
    fragment: null,
    ctx: null,
    props,
    update: noop,
    not_equal: not_equal2,
    bound: blank_object(),
    on_mount: [],
    on_destroy: [],
    on_disconnect: [],
    before_update: [],
    after_update: [],
    context: new Map(options.context || (parent_component ? parent_component.$$.context : [])),
    callbacks: blank_object(),
    dirty,
    skip_bound: false,
    root: options.target || parent_component.$$.root
  };
  append_styles && append_styles($$.root);
  let ready = false;
  $$.ctx = instance2 ? instance2(component, options.props || {}, (i, ret, ...rest) => {
    const value = rest.length ? rest[0] : ret;
    if ($$.ctx && not_equal2($$.ctx[i], $$.ctx[i] = value)) {
      if (!$$.skip_bound && $$.bound[i])
        $$.bound[i](value);
      if (ready)
        make_dirty(component, i);
    }
    return ret;
  }) : [];
  $$.update();
  ready = true;
  run_all($$.before_update);
  $$.fragment = create_fragment2 ? create_fragment2($$.ctx) : false;
  if (options.target) {
    if (options.hydrate) {
      const nodes = children(options.target);
      $$.fragment && $$.fragment.l(nodes);
      nodes.forEach(detach);
    } else {
      $$.fragment && $$.fragment.c();
    }
    if (options.intro)
      transition_in(component.$$.fragment);
    mount_component(component, options.target, options.anchor, options.customElement);
    flush();
  }
  set_current_component(parent_component);
}
class SvelteComponent {
  $destroy() {
    destroy_component(this, 1);
    this.$destroy = noop;
  }
  $on(type, callback) {
    const callbacks = this.$$.callbacks[type] || (this.$$.callbacks[type] = []);
    callbacks.push(callback);
    return () => {
      const index2 = callbacks.indexOf(callback);
      if (index2 !== -1)
        callbacks.splice(index2, 1);
    };
  }
  $set($$props) {
    if (this.$$set && !is_empty($$props)) {
      this.$$.skip_bound = true;
      this.$$set($$props);
      this.$$.skip_bound = false;
    }
  }
}
function createCodeMirror() {
  const codemirror = factory();
  usePlaceholder(codemirror);
  useOverlay(codemirror);
  useXml(codemirror);
  useMarkdown(codemirror);
  useGfm(codemirror);
  useYaml(codemirror);
  useYamlFrontmatter(codemirror);
  useContinuelist(codemirror);
  return codemirror;
}
function createEditorUtils(codemirror, editor) {
  return {
    wrapText(before, after = before) {
      const range = editor.somethingSelected() ? editor.listSelections()[0] : editor.findWordAt(editor.getCursor());
      const from = range.from();
      const to = range.to();
      const text2 = editor.getRange(from, to);
      const fromBefore = codemirror.Pos(from.line, from.ch - before.length);
      const toAfter = codemirror.Pos(to.line, to.ch + after.length);
      if (editor.getRange(fromBefore, from) === before && editor.getRange(to, toAfter) === after) {
        editor.replaceRange(text2, fromBefore, toAfter);
        editor.setSelection(fromBefore, codemirror.Pos(fromBefore.line, fromBefore.ch + text2.length));
      } else {
        editor.replaceRange(before + text2 + after, from, to);
        const cursor = editor.getCursor();
        editor.setSelection(codemirror.Pos(cursor.line, cursor.ch - after.length - text2.length), codemirror.Pos(cursor.line, cursor.ch - after.length));
      }
    },
    replaceLines(replace) {
      const [selection] = editor.listSelections();
      const range = [
        codemirror.Pos(selection.from().line, 0),
        codemirror.Pos(selection.to().line)
      ];
      const lines2 = editor.getRange(...range).split("\n");
      editor.replaceRange(lines2.map(replace).join("\n"), ...range);
      editor.setSelection(...range);
    },
    appendBlock(content) {
      const cursor = editor.getCursor();
      let emptyLine = -1;
      for (let i = cursor.line; i < editor.lineCount(); i++) {
        if (!editor.getLine(i).trim()) {
          emptyLine = i;
          break;
        }
      }
      if (emptyLine === -1) {
        editor.replaceRange("\n", codemirror.Pos(editor.lineCount()));
        emptyLine = editor.lineCount();
      }
      editor.replaceRange("\n" + content, codemirror.Pos(emptyLine));
      return codemirror.Pos(emptyLine + 1, 0);
    },
    selectFiles
  };
}
function findStartIndex(num, nums) {
  let startIndex = nums.length - 2;
  for (let i = 0; i < nums.length; i++) {
    if (num < nums[i]) {
      startIndex = i - 1;
      break;
    }
  }
  startIndex = Math.max(startIndex, 0);
  return startIndex;
}
const getShortcutWithPrefix = (key, shift = false) => {
  const shiftPrefix = shift ? "Shift-" : "";
  const CmdPrefix = typeof navigator !== "undefined" && /Mac/.test(navigator.platform) ? "Cmd-" : "Ctrl-";
  return shiftPrefix + CmdPrefix + key;
};
async function handleImageUpload({ editor, appendBlock, codemirror }, uploadImages, files) {
  const imgs = await uploadImages(files);
  const pos = appendBlock(imgs.map(({ url, alt, title }, i) => {
    alt = alt != null ? alt : files[i].name;
    return `![${alt}](${url}${title ? ` "${title}"` : ""})`;
  }).join("\n\n"));
  editor.setSelection(pos, codemirror.Pos(pos.line + imgs.length * 2 - 2));
  editor.focus();
}
function getBuiltinActions(locale, plugins, uploadImages) {
  const items = [
    {
      icon: icons.H({}),
      handler: {
        type: "dropdown",
        actions: [1, 2, 3, 4, 5, 6].map((level) => ({
          title: locale[`h${level}`],
          icon: [
            icons.H1({}),
            icons.H2({}),
            icons.H3({}),
            icons.LevelFourTitle({}),
            icons.LevelFiveTitle({}),
            icons.LevelSixTitle({})
          ][level - 1],
          cheatsheet: level <= 3 ? `${"#".repeat(level)} ${locale.headingText}` : void 0,
          handler: {
            type: "action",
            click({ replaceLines, editor }) {
              replaceLines((line) => {
                line = line.trim().replace(/^#*/, "").trim();
                line = "#".repeat(level) + " " + line;
                return line;
              });
              editor.focus();
            }
          }
        }))
      }
    },
    {
      title: locale.bold,
      icon: icons.TextBold({}),
      cheatsheet: `**${locale.boldText}**`,
      handler: {
        type: "action",
        shortcut: getShortcutWithPrefix("B"),
        click({ wrapText, editor }) {
          wrapText("**");
          editor.focus();
        }
      }
    },
    {
      title: locale.italic,
      icon: icons.TextItalic({}),
      cheatsheet: `*${locale.italicText}*`,
      handler: {
        type: "action",
        shortcut: getShortcutWithPrefix("I"),
        click({ wrapText, editor }) {
          wrapText("*");
          editor.focus();
        }
      }
    },
    {
      title: locale.quote,
      icon: icons.Quote({}),
      cheatsheet: `> ${locale.quotedText}`,
      handler: {
        type: "action",
        click({ replaceLines, editor }) {
          replaceLines((line) => "> " + line);
          editor.focus();
        }
      }
    },
    {
      title: locale.link,
      icon: icons.LinkOne({}),
      cheatsheet: `[${locale.linkText}](url)`,
      handler: {
        type: "action",
        shortcut: getShortcutWithPrefix("K"),
        click({ editor, wrapText, codemirror }) {
          wrapText("[", "](url)");
          const cursor = editor.getCursor();
          editor.setSelection(codemirror.Pos(cursor.line, cursor.ch + 2), codemirror.Pos(cursor.line, cursor.ch + 5));
          editor.focus();
        }
      }
    },
    {
      title: locale.image,
      icon: icons.Pic({}),
      cheatsheet: `![${locale.imageAlt}](url "${locale.imageTitle}")`,
      handler: uploadImages ? {
        type: "action",
        shortcut: getShortcutWithPrefix("I", true),
        async click(ctx) {
          const fileList = await selectFiles({
            accept: "image/*",
            multiple: true
          });
          if (fileList == null ? void 0 : fileList.length) {
            await handleImageUpload(ctx, uploadImages, Array.from(fileList));
          }
        }
      } : void 0
    },
    {
      title: locale.code,
      icon: icons.Code({}),
      cheatsheet: "`" + locale.codeText + "`",
      handler: {
        type: "action",
        shortcut: getShortcutWithPrefix("K", true),
        click({ wrapText, editor }) {
          wrapText("`");
          editor.focus();
        }
      }
    },
    {
      title: locale.codeBlock,
      icon: icons.CodeBrackets({}),
      cheatsheet: "```" + locale.codeLang + "\u21B5",
      handler: {
        type: "action",
        shortcut: getShortcutWithPrefix("C", true),
        click({ editor, appendBlock, codemirror }) {
          const pos = appendBlock("```js\n```");
          editor.setSelection(codemirror.Pos(pos.line, 3), codemirror.Pos(pos.line, 5));
          editor.focus();
        }
      }
    },
    {
      title: locale.ul,
      icon: icons.ListTwo({}),
      cheatsheet: `- ${locale.ulItem}`,
      handler: {
        type: "action",
        shortcut: getShortcutWithPrefix("U", true),
        click({ replaceLines, editor }) {
          replaceLines((line) => "- " + line);
          editor.focus();
        }
      }
    },
    {
      title: locale.ol,
      icon: icons.OrderedList({}),
      cheatsheet: `1. ${locale.olItem}`,
      handler: {
        type: "action",
        shortcut: getShortcutWithPrefix("O", true),
        click({ replaceLines, editor }) {
          replaceLines((line, i) => `${i + 1}. ${line}`);
          editor.focus();
        }
      }
    },
    {
      title: locale.hr,
      icon: icons.DividingLine({}),
      cheatsheet: "---"
    }
  ];
  plugins.forEach(({ actions }) => {
    if (actions)
      items.push(...actions);
  });
  return items;
}
function get_each_context$2(ctx, list, i) {
  const child_ctx = ctx.slice();
  child_ctx[21] = list[i];
  child_ctx[23] = i;
  return child_ctx;
}
function get_each_context_1$1(ctx, list, i) {
  const child_ctx = ctx.slice();
  child_ctx[21] = list[i];
  child_ctx[23] = i;
  return child_ctx;
}
function create_else_block(ctx) {
  let div0;
  let t0_value = ctx[2].write + "";
  let t0;
  let div1;
  let t1_value = ctx[2].preview + "";
  let t1;
  let mounted;
  let dispose;
  return {
    c() {
      div0 = element("div");
      t0 = text(t0_value);
      div1 = element("div");
      t1 = text(t1_value);
      attr(div0, "class", "bytemd-toolbar-tab");
      toggle_class(div0, "bytemd-toolbar-tab-active", ctx[1] !== "preview");
      attr(div1, "class", "bytemd-toolbar-tab");
      toggle_class(div1, "bytemd-toolbar-tab-active", ctx[1] === "preview");
    },
    m(target, anchor) {
      insert(target, div0, anchor);
      append(div0, t0);
      insert(target, div1, anchor);
      append(div1, t1);
      if (!mounted) {
        dispose = [
          listen(div0, "click", ctx[15]),
          listen(div1, "click", ctx[16])
        ];
        mounted = true;
      }
    },
    p(ctx2, dirty) {
      if (dirty & 4 && t0_value !== (t0_value = ctx2[2].write + ""))
        set_data(t0, t0_value);
      if (dirty & 2) {
        toggle_class(div0, "bytemd-toolbar-tab-active", ctx2[1] !== "preview");
      }
      if (dirty & 4 && t1_value !== (t1_value = ctx2[2].preview + ""))
        set_data(t1, t1_value);
      if (dirty & 2) {
        toggle_class(div1, "bytemd-toolbar-tab-active", ctx2[1] === "preview");
      }
    },
    d(detaching) {
      if (detaching)
        detach(div0);
      if (detaching)
        detach(div1);
      mounted = false;
      run_all(dispose);
    }
  };
}
function create_if_block_1$2(ctx) {
  let each_1_anchor;
  let each_value_1 = ctx[3];
  let each_blocks = [];
  for (let i = 0; i < each_value_1.length; i += 1) {
    each_blocks[i] = create_each_block_1$1(get_each_context_1$1(ctx, each_value_1, i));
  }
  return {
    c() {
      for (let i = 0; i < each_blocks.length; i += 1) {
        each_blocks[i].c();
      }
      each_1_anchor = empty();
    },
    m(target, anchor) {
      for (let i = 0; i < each_blocks.length; i += 1) {
        each_blocks[i].m(target, anchor);
      }
      insert(target, each_1_anchor, anchor);
    },
    p(ctx2, dirty) {
      if (dirty & 8) {
        each_value_1 = ctx2[3];
        let i;
        for (i = 0; i < each_value_1.length; i += 1) {
          const child_ctx = get_each_context_1$1(ctx2, each_value_1, i);
          if (each_blocks[i]) {
            each_blocks[i].p(child_ctx, dirty);
          } else {
            each_blocks[i] = create_each_block_1$1(child_ctx);
            each_blocks[i].c();
            each_blocks[i].m(each_1_anchor.parentNode, each_1_anchor);
          }
        }
        for (; i < each_blocks.length; i += 1) {
          each_blocks[i].d(1);
        }
        each_blocks.length = each_value_1.length;
      }
    },
    d(detaching) {
      destroy_each(each_blocks, detaching);
      if (detaching)
        detach(each_1_anchor);
    }
  };
}
function create_if_block_2(ctx) {
  let div;
  let raw_value = ctx[21].icon + "";
  let div_class_value;
  let div_bytemd_tippy_path_value;
  return {
    c() {
      div = element("div");
      attr(div, "class", div_class_value = ["bytemd-toolbar-icon", tippyClass].join(" "));
      attr(div, "bytemd-tippy-path", div_bytemd_tippy_path_value = ctx[23]);
    },
    m(target, anchor) {
      insert(target, div, anchor);
      div.innerHTML = raw_value;
    },
    p(ctx2, dirty) {
      if (dirty & 8 && raw_value !== (raw_value = ctx2[21].icon + ""))
        div.innerHTML = raw_value;
    },
    d(detaching) {
      if (detaching)
        detach(div);
    }
  };
}
function create_each_block_1$1(ctx) {
  let if_block_anchor;
  let if_block = ctx[21].handler && create_if_block_2(ctx);
  return {
    c() {
      if (if_block)
        if_block.c();
      if_block_anchor = empty();
    },
    m(target, anchor) {
      if (if_block)
        if_block.m(target, anchor);
      insert(target, if_block_anchor, anchor);
    },
    p(ctx2, dirty) {
      if (ctx2[21].handler) {
        if (if_block) {
          if_block.p(ctx2, dirty);
        } else {
          if_block = create_if_block_2(ctx2);
          if_block.c();
          if_block.m(if_block_anchor.parentNode, if_block_anchor);
        }
      } else if (if_block) {
        if_block.d(1);
        if_block = null;
      }
    },
    d(detaching) {
      if (if_block)
        if_block.d(detaching);
      if (detaching)
        detach(if_block_anchor);
    }
  };
}
function create_if_block$3(ctx) {
  let div;
  let raw_value = ctx[21].icon + "";
  let div_class_value;
  let div_bytemd_tippy_path_value;
  return {
    c() {
      div = element("div");
      attr(div, "class", div_class_value = ["bytemd-toolbar-icon", tippyClass, tippyClassRight].join(" "));
      attr(div, "bytemd-tippy-path", div_bytemd_tippy_path_value = ctx[23]);
      toggle_class(div, "bytemd-toolbar-icon-active", ctx[21].active);
    },
    m(target, anchor) {
      insert(target, div, anchor);
      div.innerHTML = raw_value;
    },
    p(ctx2, dirty) {
      if (dirty & 32 && raw_value !== (raw_value = ctx2[21].icon + ""))
        div.innerHTML = raw_value;
      if (dirty & 32) {
        toggle_class(div, "bytemd-toolbar-icon-active", ctx2[21].active);
      }
    },
    d(detaching) {
      if (detaching)
        detach(div);
    }
  };
}
function create_each_block$2(ctx) {
  let if_block_anchor;
  let if_block = !ctx[21].hidden && create_if_block$3(ctx);
  return {
    c() {
      if (if_block)
        if_block.c();
      if_block_anchor = empty();
    },
    m(target, anchor) {
      if (if_block)
        if_block.m(target, anchor);
      insert(target, if_block_anchor, anchor);
    },
    p(ctx2, dirty) {
      if (!ctx2[21].hidden) {
        if (if_block) {
          if_block.p(ctx2, dirty);
        } else {
          if_block = create_if_block$3(ctx2);
          if_block.c();
          if_block.m(if_block_anchor.parentNode, if_block_anchor);
        }
      } else if (if_block) {
        if_block.d(1);
        if_block = null;
      }
    },
    d(detaching) {
      if (if_block)
        if_block.d(detaching);
      if (detaching)
        detach(if_block_anchor);
    }
  };
}
function create_fragment$5(ctx) {
  let div2;
  let div0;
  let div1;
  let mounted;
  let dispose;
  function select_block_type(ctx2, dirty) {
    if (ctx2[0])
      return create_if_block_1$2;
    return create_else_block;
  }
  let current_block_type = select_block_type(ctx);
  let if_block = current_block_type(ctx);
  let each_value = ctx[5];
  let each_blocks = [];
  for (let i = 0; i < each_value.length; i += 1) {
    each_blocks[i] = create_each_block$2(get_each_context$2(ctx, each_value, i));
  }
  return {
    c() {
      div2 = element("div");
      div0 = element("div");
      if_block.c();
      div1 = element("div");
      for (let i = 0; i < each_blocks.length; i += 1) {
        each_blocks[i].c();
      }
      attr(div0, "class", "bytemd-toolbar-left");
      attr(div1, "class", "bytemd-toolbar-right");
      attr(div2, "class", "bytemd-toolbar");
    },
    m(target, anchor) {
      insert(target, div2, anchor);
      append(div2, div0);
      if_block.m(div0, null);
      append(div2, div1);
      for (let i = 0; i < each_blocks.length; i += 1) {
        each_blocks[i].m(div1, null);
      }
      ctx[17](div2);
      if (!mounted) {
        dispose = listen(div2, "click", ctx[7]);
        mounted = true;
      }
    },
    p(ctx2, [dirty]) {
      if (current_block_type === (current_block_type = select_block_type(ctx2)) && if_block) {
        if_block.p(ctx2, dirty);
      } else {
        if_block.d(1);
        if_block = current_block_type(ctx2);
        if (if_block) {
          if_block.c();
          if_block.m(div0, null);
        }
      }
      if (dirty & 32) {
        each_value = ctx2[5];
        let i;
        for (i = 0; i < each_value.length; i += 1) {
          const child_ctx = get_each_context$2(ctx2, each_value, i);
          if (each_blocks[i]) {
            each_blocks[i].p(child_ctx, dirty);
          } else {
            each_blocks[i] = create_each_block$2(child_ctx);
            each_blocks[i].c();
            each_blocks[i].m(div1, null);
          }
        }
        for (; i < each_blocks.length; i += 1) {
          each_blocks[i].d(1);
        }
        each_blocks.length = each_value.length;
      }
    },
    i: noop,
    o: noop,
    d(detaching) {
      if (detaching)
        detach(div2);
      if_block.d();
      destroy_each(each_blocks, detaching);
      ctx[17](null);
      mounted = false;
      dispose();
    }
  };
}
const tippyClass = "bytemd-tippy";
const tippyClassRight = "bytemd-tippy-right";
const tippyPathKey = "bytemd-tippy-path";
function instance$5($$self, $$props, $$invalidate) {
  let tocActive;
  let helpActive;
  let writeActive;
  let previewActive;
  let rightActions;
  const dispatch = createEventDispatcher();
  let toolbar;
  let { context } = $$props;
  let { split } = $$props;
  let { activeTab } = $$props;
  let { fullscreen: fullscreen2 } = $$props;
  let { sidebar } = $$props;
  let { locale } = $$props;
  let { actions } = $$props;
  function getPayloadFromElement(e) {
    var _a, _b;
    const paths = (_b = (_a = e.getAttribute(tippyPathKey)) == null ? void 0 : _a.split("-")) == null ? void 0 : _b.map((x) => parseInt(x, 10));
    if (!paths)
      return;
    let item = {
      title: "",
      handler: {
        type: "dropdown",
        actions: e.classList.contains(tippyClassRight) ? rightActions : actions
      }
    };
    paths == null ? void 0 : paths.forEach((index2) => {
      var _a2;
      if (((_a2 = item.handler) == null ? void 0 : _a2.type) === "dropdown") {
        item = item.handler.actions[index2];
      }
    });
    return { paths, item };
  }
  let delegateInstance;
  function init2() {
    delegateInstance = delegate(toolbar, {
      target: `.${tippyClass}`,
      onCreate({ setProps, reference }) {
        const payload = getPayloadFromElement(reference);
        if (!payload)
          return;
        const { item, paths } = payload;
        const { handler } = item;
        if (!handler)
          return;
        if (handler.type === "action") {
          setProps({
            content: item.title,
            onHidden(ins) {
              ins.destroy();
            }
          });
        } else if (handler.type === "dropdown") {
          const dropdown = document.createElement("div");
          dropdown.classList.add("bytemd-dropdown");
          if (item.title) {
            const dropdownTitle = document.createElement("div");
            dropdownTitle.classList.add("bytemd-dropdown-title");
            dropdownTitle.appendChild(document.createTextNode(item.title));
            dropdown.appendChild(dropdownTitle);
          }
          handler.actions.forEach((subAction, i) => {
            var _a;
            const dropdownItem = document.createElement("div");
            dropdownItem.classList.add("bytemd-dropdown-item");
            dropdownItem.setAttribute(tippyPathKey, [...paths, i].join("-"));
            if (((_a = subAction.handler) == null ? void 0 : _a.type) === "dropdown") {
              dropdownItem.classList.add(tippyClass);
            }
            if (reference.classList.contains(tippyClassRight)) {
              dropdownItem.classList.add(tippyClassRight);
            }
            dropdownItem.innerHTML = `${subAction.icon ? `<div class="bytemd-dropdown-item-icon">${subAction.icon}</div>` : ""}<div class="bytemd-dropdown-item-title">${subAction.title}</div>`;
            dropdown.appendChild(dropdownItem);
          });
          setProps({
            allowHTML: true,
            showOnCreate: true,
            theme: "light-border",
            placement: "bottom-start",
            interactive: true,
            interactiveDebounce: 50,
            arrow: false,
            offset: [0, 4],
            content: dropdown.outerHTML,
            onHidden(ins) {
              ins.destroy();
            },
            onCreate(ins) {
              [...ins.popper.querySelectorAll(".bytemd-dropdown-item")].forEach((el, i) => {
                var _a;
                const actionHandler = (_a = handler.actions[i]) == null ? void 0 : _a.handler;
                if ((actionHandler == null ? void 0 : actionHandler.type) === "action") {
                  const { mouseenter, mouseleave } = actionHandler;
                  if (mouseenter) {
                    el.addEventListener("mouseenter", () => {
                      mouseenter(context);
                    });
                  }
                  if (mouseleave) {
                    el.addEventListener("mouseleave", () => {
                      mouseleave(context);
                    });
                  }
                }
              });
            }
          });
        }
      }
    });
  }
  onMount(() => {
    init2();
  });
  function handleClick(e) {
    var _a, _b;
    const target = e.target.closest(`[${tippyPathKey}]`);
    if (!target)
      return;
    const handler = (_b = (_a = getPayloadFromElement(target)) == null ? void 0 : _a.item) == null ? void 0 : _b.handler;
    if ((handler == null ? void 0 : handler.type) === "action") {
      handler.click(context);
    }
    delegateInstance == null ? void 0 : delegateInstance.destroy();
    init2();
  }
  const click_handler = () => dispatch("tab", "write");
  const click_handler_1 = () => dispatch("tab", "preview");
  function div2_binding($$value) {
    binding_callbacks[$$value ? "unshift" : "push"](() => {
      toolbar = $$value;
      $$invalidate(4, toolbar);
    });
  }
  $$self.$$set = ($$props2) => {
    if ("context" in $$props2)
      $$invalidate(8, context = $$props2.context);
    if ("split" in $$props2)
      $$invalidate(0, split = $$props2.split);
    if ("activeTab" in $$props2)
      $$invalidate(1, activeTab = $$props2.activeTab);
    if ("fullscreen" in $$props2)
      $$invalidate(9, fullscreen2 = $$props2.fullscreen);
    if ("sidebar" in $$props2)
      $$invalidate(10, sidebar = $$props2.sidebar);
    if ("locale" in $$props2)
      $$invalidate(2, locale = $$props2.locale);
    if ("actions" in $$props2)
      $$invalidate(3, actions = $$props2.actions);
  };
  $$self.$$.update = () => {
    if ($$self.$$.dirty & 1024) {
      $$invalidate(14, tocActive = sidebar === "toc");
    }
    if ($$self.$$.dirty & 1024) {
      $$invalidate(13, helpActive = sidebar === "help");
    }
    if ($$self.$$.dirty & 2) {
      $$invalidate(12, writeActive = activeTab === "write");
    }
    if ($$self.$$.dirty & 2) {
      $$invalidate(11, previewActive = activeTab === "preview");
    }
    if ($$self.$$.dirty & 31237) {
      $$invalidate(5, rightActions = [
        {
          title: tocActive ? locale.closeToc : locale.toc,
          icon: icons.AlignTextLeftOne({}),
          handler: {
            type: "action",
            click() {
              dispatch("click", "toc");
            }
          },
          active: tocActive
        },
        {
          title: helpActive ? locale.closeHelp : locale.help,
          icon: icons.Helpcenter({}),
          handler: {
            type: "action",
            click() {
              dispatch("click", "help");
            }
          },
          active: helpActive
        },
        {
          title: writeActive ? locale.exitWriteOnly : locale.writeOnly,
          icon: icons.LeftExpand({}),
          handler: {
            type: "action",
            click() {
              dispatch("tab", "write");
            }
          },
          active: writeActive,
          hidden: !split
        },
        {
          title: previewActive ? locale.exitPreviewOnly : locale.previewOnly,
          icon: icons.RightExpand({}),
          handler: {
            type: "action",
            click() {
              dispatch("tab", "preview");
            }
          },
          active: previewActive,
          hidden: !split
        },
        {
          title: fullscreen2 ? locale.exitFullscreen : locale.fullscreen,
          icon: fullscreen2 ? icons.OffScreen({}) : icons.FullScreen({}),
          handler: {
            type: "action",
            click() {
              dispatch("click", "fullscreen");
            }
          }
        },
        {
          title: locale.source,
          icon: icons.GithubOne({}),
          handler: {
            type: "action",
            click() {
              window.open("https://github.com/bytedance/bytemd");
            }
          }
        }
      ]);
    }
  };
  return [
    split,
    activeTab,
    locale,
    actions,
    toolbar,
    rightActions,
    dispatch,
    handleClick,
    context,
    fullscreen2,
    sidebar,
    previewActive,
    writeActive,
    helpActive,
    tocActive,
    click_handler,
    click_handler_1,
    div2_binding
  ];
}
class Toolbar extends SvelteComponent {
  constructor(options) {
    super();
    init(this, options, instance$5, create_fragment$5, not_equal, {
      context: 8,
      split: 0,
      activeTab: 1,
      fullscreen: 9,
      sidebar: 10,
      locale: 2,
      actions: 3
    });
  }
}
const schemaStr = JSON.stringify(defaultSchema);
function getProcessor({
  sanitize,
  plugins,
  remarkRehype: remarkRehypeOptions = {}
}) {
  let processor = unified().use(remarkParse);
  plugins == null ? void 0 : plugins.forEach(({ remark }) => {
    if (remark)
      processor = remark(processor);
  });
  processor = processor.use(remarkRehype, { allowDangerousHtml: true, ...remarkRehypeOptions }).use(rehypeRaw);
  let schema = JSON.parse(schemaStr);
  schema.attributes["*"].push("className");
  if (typeof sanitize === "function") {
    schema = sanitize(schema);
  }
  processor = processor.use(rehypeSanitize, schema);
  plugins == null ? void 0 : plugins.forEach(({ rehype }) => {
    if (rehype)
      processor = rehype(processor);
  });
  return processor.use(rehypeStringify);
}
function create_fragment$4(ctx) {
  let div;
  return {
    c() {
      div = element("div");
      attr(div, "class", "markdown-body");
    },
    m(target, anchor) {
      insert(target, div, anchor);
      div.innerHTML = ctx[1];
      ctx[8](div);
    },
    p(ctx2, [dirty]) {
      if (dirty & 2)
        div.innerHTML = ctx2[1];
    },
    i: noop,
    o: noop,
    d(detaching) {
      if (detaching)
        detach(div);
      ctx[8](null);
    }
  };
}
function instance$4($$self, $$props, $$invalidate) {
  let html;
  const dispatch = createEventDispatcher();
  let { value = "" } = $$props;
  let { plugins = [] } = $$props;
  let { sanitize = void 0 } = $$props;
  let { remarkRehype: remarkRehype2 = void 0 } = $$props;
  let markdownBody;
  let cbs = [];
  function on() {
    cbs = plugins.map((p) => {
      var _a;
      return (_a = p.viewerEffect) == null ? void 0 : _a.call(p, { markdownBody, file });
    });
  }
  function off() {
    cbs.forEach((cb) => cb == null ? void 0 : cb());
  }
  onMount(() => {
    markdownBody.addEventListener("click", (e) => {
      var _a;
      const $ = e.target;
      if ($.tagName !== "A")
        return;
      const href = $.getAttribute("href");
      if (!(href == null ? void 0 : href.startsWith("#")))
        return;
      (_a = markdownBody.querySelector("#user-content-" + href.slice(1))) == null ? void 0 : _a.scrollIntoView();
    });
  });
  onDestroy(off);
  let file;
  let i = 0;
  const dispatchPlugin = () => (tree, file2) => {
    tick().then(() => {
      dispatch("hast", { hast: tree, file: file2 });
    });
  };
  afterUpdate(() => {
    off();
    on();
  });
  function div_binding($$value) {
    binding_callbacks[$$value ? "unshift" : "push"](() => {
      markdownBody = $$value;
      $$invalidate(0, markdownBody);
    });
  }
  $$self.$$set = ($$props2) => {
    if ("value" in $$props2)
      $$invalidate(2, value = $$props2.value);
    if ("plugins" in $$props2)
      $$invalidate(3, plugins = $$props2.plugins);
    if ("sanitize" in $$props2)
      $$invalidate(4, sanitize = $$props2.sanitize);
    if ("remarkRehype" in $$props2)
      $$invalidate(5, remarkRehype2 = $$props2.remarkRehype);
  };
  $$self.$$.update = () => {
    if ($$self.$$.dirty & 188) {
      try {
        $$invalidate(6, file = getProcessor({
          sanitize,
          plugins: [
            ...plugins,
            {
              rehype: (processor) => processor.use(dispatchPlugin)
            }
          ],
          remarkRehype: remarkRehype2
        }).processSync(value));
        $$invalidate(7, i++, i);
      } catch (err) {
        console.error(err);
      }
    }
    if ($$self.$$.dirty & 192) {
      $$invalidate(1, html = `${file}<!--${i}-->`);
    }
  };
  return [
    markdownBody,
    html,
    value,
    plugins,
    sanitize,
    remarkRehype2,
    file,
    i,
    div_binding
  ];
}
class Viewer extends SvelteComponent {
  constructor(options) {
    super();
    init(this, options, instance$4, create_fragment$4, not_equal, {
      value: 2,
      plugins: 3,
      sanitize: 4,
      remarkRehype: 5
    });
  }
}
function get_each_context$1(ctx, list, i) {
  const child_ctx = ctx.slice();
  child_ctx[10] = list[i];
  child_ctx[12] = i;
  return child_ctx;
}
function create_each_block$1(ctx) {
  let li;
  let t_value = ctx[10].text + "";
  let t;
  let li_class_value;
  let li_style_value;
  let mounted;
  let dispose;
  function click_handler() {
    return ctx[8](ctx[12]);
  }
  return {
    c() {
      li = element("li");
      t = text(t_value);
      attr(li, "class", li_class_value = `bytemd-toc-${ctx[10].level}`);
      attr(li, "style", li_style_value = `padding-left:${(ctx[10].level - ctx[3]) * 16 + 8}px`);
      toggle_class(li, "bytemd-toc-active", ctx[4] === ctx[12]);
      toggle_class(li, "bytemd-toc-first", ctx[10].level === ctx[3]);
    },
    m(target, anchor) {
      insert(target, li, anchor);
      append(li, t);
      if (!mounted) {
        dispose = listen(li, "click", click_handler);
        mounted = true;
      }
    },
    p(new_ctx, dirty) {
      ctx = new_ctx;
      if (dirty & 4 && t_value !== (t_value = ctx[10].text + ""))
        set_data(t, t_value);
      if (dirty & 4 && li_class_value !== (li_class_value = `bytemd-toc-${ctx[10].level}`)) {
        attr(li, "class", li_class_value);
      }
      if (dirty & 12 && li_style_value !== (li_style_value = `padding-left:${(ctx[10].level - ctx[3]) * 16 + 8}px`)) {
        attr(li, "style", li_style_value);
      }
      if (dirty & 20) {
        toggle_class(li, "bytemd-toc-active", ctx[4] === ctx[12]);
      }
      if (dirty & 12) {
        toggle_class(li, "bytemd-toc-first", ctx[10].level === ctx[3]);
      }
    },
    d(detaching) {
      if (detaching)
        detach(li);
      mounted = false;
      dispose();
    }
  };
}
function create_fragment$3(ctx) {
  let div;
  let h22;
  let t_value = ctx[0].toc + "";
  let t;
  let ul2;
  let each_value = ctx[2];
  let each_blocks = [];
  for (let i = 0; i < each_value.length; i += 1) {
    each_blocks[i] = create_each_block$1(get_each_context$1(ctx, each_value, i));
  }
  return {
    c() {
      div = element("div");
      h22 = element("h2");
      t = text(t_value);
      ul2 = element("ul");
      for (let i = 0; i < each_blocks.length; i += 1) {
        each_blocks[i].c();
      }
      attr(div, "class", "bytemd-toc");
      toggle_class(div, "bytemd-hidden", !ctx[1]);
    },
    m(target, anchor) {
      insert(target, div, anchor);
      append(div, h22);
      append(h22, t);
      append(div, ul2);
      for (let i = 0; i < each_blocks.length; i += 1) {
        each_blocks[i].m(ul2, null);
      }
    },
    p(ctx2, [dirty]) {
      if (dirty & 1 && t_value !== (t_value = ctx2[0].toc + ""))
        set_data(t, t_value);
      if (dirty & 60) {
        each_value = ctx2[2];
        let i;
        for (i = 0; i < each_value.length; i += 1) {
          const child_ctx = get_each_context$1(ctx2, each_value, i);
          if (each_blocks[i]) {
            each_blocks[i].p(child_ctx, dirty);
          } else {
            each_blocks[i] = create_each_block$1(child_ctx);
            each_blocks[i].c();
            each_blocks[i].m(ul2, null);
          }
        }
        for (; i < each_blocks.length; i += 1) {
          each_blocks[i].d(1);
        }
        each_blocks.length = each_value.length;
      }
      if (dirty & 2) {
        toggle_class(div, "bytemd-hidden", !ctx2[1]);
      }
    },
    i: noop,
    o: noop,
    d(detaching) {
      if (detaching)
        detach(div);
      destroy_each(each_blocks, detaching);
    }
  };
}
function instance$3($$self, $$props, $$invalidate) {
  let { hast } = $$props;
  let { currentBlockIndex } = $$props;
  let { locale } = $$props;
  let { visible } = $$props;
  const dispatch = createEventDispatcher();
  let items;
  let minLevel = 6;
  let currentHeadingIndex = 0;
  function stringifyHeading(e) {
    let result = "";
    visit(e, (node) => {
      if (node.type === "text") {
        result += node.value;
      }
    });
    return result;
  }
  const click_handler = (index2) => {
    dispatch("click", index2);
  };
  $$self.$$set = ($$props2) => {
    if ("hast" in $$props2)
      $$invalidate(6, hast = $$props2.hast);
    if ("currentBlockIndex" in $$props2)
      $$invalidate(7, currentBlockIndex = $$props2.currentBlockIndex);
    if ("locale" in $$props2)
      $$invalidate(0, locale = $$props2.locale);
    if ("visible" in $$props2)
      $$invalidate(1, visible = $$props2.visible);
  };
  $$self.$$.update = () => {
    if ($$self.$$.dirty & 204) {
      (() => {
        $$invalidate(2, items = []);
        $$invalidate(4, currentHeadingIndex = 0);
        hast.children.filter((v) => v.type === "element").forEach((node, index2) => {
          if (node.tagName[0] === "h" && !!node.children.length) {
            const i = Number(node.tagName[1]);
            $$invalidate(3, minLevel = Math.min(minLevel, i));
            items.push({ level: i, text: stringifyHeading(node) });
          }
          if (currentBlockIndex >= index2) {
            $$invalidate(4, currentHeadingIndex = items.length - 1);
          }
        });
      })();
    }
  };
  return [
    locale,
    visible,
    items,
    minLevel,
    currentHeadingIndex,
    dispatch,
    hast,
    currentBlockIndex,
    click_handler
  ];
}
class Toc extends SvelteComponent {
  constructor(options) {
    super();
    init(this, options, instance$3, create_fragment$3, not_equal, {
      hast: 6,
      currentBlockIndex: 7,
      locale: 0,
      visible: 1
    });
  }
}
function create_if_block_1$1(ctx) {
  let span;
  let t_value = ctx[2].limited + "";
  let t;
  return {
    c() {
      span = element("span");
      t = text(t_value);
      attr(span, "class", "bytemd-status-error");
    },
    m(target, anchor) {
      insert(target, span, anchor);
      append(span, t);
    },
    p(ctx2, dirty) {
      if (dirty & 4 && t_value !== (t_value = ctx2[2].limited + ""))
        set_data(t, t_value);
    },
    d(detaching) {
      if (detaching)
        detach(span);
    }
  };
}
function create_if_block$2(ctx) {
  let label;
  let input;
  let t_value = ctx[2].sync + "";
  let t;
  let mounted;
  let dispose;
  return {
    c() {
      label = element("label");
      input = element("input");
      t = text(t_value);
      attr(input, "type", "checkbox");
      input.checked = ctx[1];
    },
    m(target, anchor) {
      insert(target, label, anchor);
      append(label, input);
      append(label, t);
      if (!mounted) {
        dispose = listen(input, "change", ctx[8]);
        mounted = true;
      }
    },
    p(ctx2, dirty) {
      if (dirty & 2) {
        input.checked = ctx2[1];
      }
      if (dirty & 4 && t_value !== (t_value = ctx2[2].sync + ""))
        set_data(t, t_value);
    },
    d(detaching) {
      if (detaching)
        detach(label);
      mounted = false;
      dispose();
    }
  };
}
function create_fragment$2(ctx) {
  let div2;
  let div0;
  let span0;
  let t0_value = ctx[2].words + "";
  let t0;
  let t1;
  let strong0;
  let t2;
  let span1;
  let t3_value = ctx[2].lines + "";
  let t3;
  let t4;
  let strong1;
  let t5;
  let div1;
  let span2;
  let t6_value = ctx[2].top + "";
  let t6;
  let mounted;
  let dispose;
  let if_block0 = ctx[3] && create_if_block_1$1(ctx);
  let if_block1 = ctx[0] && create_if_block$2(ctx);
  return {
    c() {
      div2 = element("div");
      div0 = element("div");
      span0 = element("span");
      t0 = text(t0_value);
      t1 = text(": ");
      strong0 = element("strong");
      t2 = text(ctx[5]);
      span1 = element("span");
      t3 = text(t3_value);
      t4 = text(": ");
      strong1 = element("strong");
      t5 = text(ctx[4]);
      if (if_block0)
        if_block0.c();
      div1 = element("div");
      if (if_block1)
        if_block1.c();
      span2 = element("span");
      t6 = text(t6_value);
      attr(div0, "class", "bytemd-status-left");
      attr(div1, "class", "bytemd-status-right");
      attr(div2, "class", "bytemd-status");
    },
    m(target, anchor) {
      insert(target, div2, anchor);
      append(div2, div0);
      append(div0, span0);
      append(span0, t0);
      append(span0, t1);
      append(span0, strong0);
      append(strong0, t2);
      append(div0, span1);
      append(span1, t3);
      append(span1, t4);
      append(span1, strong1);
      append(strong1, t5);
      if (if_block0)
        if_block0.m(div0, null);
      append(div2, div1);
      if (if_block1)
        if_block1.m(div1, null);
      append(div1, span2);
      append(span2, t6);
      if (!mounted) {
        dispose = listen(span2, "click", ctx[9]);
        mounted = true;
      }
    },
    p(ctx2, [dirty]) {
      if (dirty & 4 && t0_value !== (t0_value = ctx2[2].words + ""))
        set_data(t0, t0_value);
      if (dirty & 32)
        set_data(t2, ctx2[5]);
      if (dirty & 4 && t3_value !== (t3_value = ctx2[2].lines + ""))
        set_data(t3, t3_value);
      if (dirty & 16)
        set_data(t5, ctx2[4]);
      if (ctx2[3]) {
        if (if_block0) {
          if_block0.p(ctx2, dirty);
        } else {
          if_block0 = create_if_block_1$1(ctx2);
          if_block0.c();
          if_block0.m(div0, null);
        }
      } else if (if_block0) {
        if_block0.d(1);
        if_block0 = null;
      }
      if (ctx2[0]) {
        if (if_block1) {
          if_block1.p(ctx2, dirty);
        } else {
          if_block1 = create_if_block$2(ctx2);
          if_block1.c();
          if_block1.m(div1, span2);
        }
      } else if (if_block1) {
        if_block1.d(1);
        if_block1 = null;
      }
      if (dirty & 4 && t6_value !== (t6_value = ctx2[2].top + ""))
        set_data(t6, t6_value);
    },
    i: noop,
    o: noop,
    d(detaching) {
      if (detaching)
        detach(div2);
      if (if_block0)
        if_block0.d();
      if (if_block1)
        if_block1.d();
      mounted = false;
      dispose();
    }
  };
}
function instance$2($$self, $$props, $$invalidate) {
  let words2;
  let lines2;
  let { showSync } = $$props;
  let { value } = $$props;
  let { syncEnabled } = $$props;
  let { locale } = $$props;
  let { islimited } = $$props;
  const dispatch = createEventDispatcher();
  const change_handler = () => dispatch("sync", !syncEnabled);
  const click_handler = () => dispatch("top");
  $$self.$$set = ($$props2) => {
    if ("showSync" in $$props2)
      $$invalidate(0, showSync = $$props2.showSync);
    if ("value" in $$props2)
      $$invalidate(7, value = $$props2.value);
    if ("syncEnabled" in $$props2)
      $$invalidate(1, syncEnabled = $$props2.syncEnabled);
    if ("locale" in $$props2)
      $$invalidate(2, locale = $$props2.locale);
    if ("islimited" in $$props2)
      $$invalidate(3, islimited = $$props2.islimited);
  };
  $$self.$$.update = () => {
    if ($$self.$$.dirty & 128) {
      $$invalidate(5, words2 = wordCount(value));
    }
    if ($$self.$$.dirty & 128) {
      $$invalidate(4, lines2 = value.split("\n").length);
    }
  };
  return [
    showSync,
    syncEnabled,
    locale,
    islimited,
    lines2,
    words2,
    dispatch,
    value,
    change_handler,
    click_handler
  ];
}
class Status extends SvelteComponent {
  constructor(options) {
    super();
    init(this, options, instance$2, create_fragment$2, safe_not_equal, {
      showSync: 0,
      value: 7,
      syncEnabled: 1,
      locale: 2,
      islimited: 3
    });
  }
}
function get_each_context(ctx, list, i) {
  const child_ctx = ctx.slice();
  child_ctx[5] = list[i];
  return child_ctx;
}
function get_each_context_1(ctx, list, i) {
  const child_ctx = ctx.slice();
  child_ctx[5] = list[i];
  return child_ctx;
}
function create_if_block_1(ctx) {
  let li;
  let div0;
  let raw_value = ctx[5].icon + "";
  let div1;
  let t0_value = ctx[5].title + "";
  let t0;
  let div2;
  let code2;
  let t1_value = ctx[5].cheatsheet + "";
  let t1;
  return {
    c() {
      li = element("li");
      div0 = element("div");
      div1 = element("div");
      t0 = text(t0_value);
      div2 = element("div");
      code2 = element("code");
      t1 = text(t1_value);
      attr(div0, "class", "bytemd-help-icon");
      attr(div1, "class", "bytemd-help-title");
      attr(div2, "class", "bytemd-help-content");
    },
    m(target, anchor) {
      insert(target, li, anchor);
      append(li, div0);
      div0.innerHTML = raw_value;
      append(li, div1);
      append(div1, t0);
      append(li, div2);
      append(div2, code2);
      append(code2, t1);
    },
    p(ctx2, dirty) {
      if (dirty & 4 && raw_value !== (raw_value = ctx2[5].icon + ""))
        div0.innerHTML = raw_value;
      if (dirty & 4 && t0_value !== (t0_value = ctx2[5].title + ""))
        set_data(t0, t0_value);
      if (dirty & 4 && t1_value !== (t1_value = ctx2[5].cheatsheet + ""))
        set_data(t1, t1_value);
    },
    d(detaching) {
      if (detaching)
        detach(li);
    }
  };
}
function create_each_block_1(ctx) {
  let if_block_anchor;
  let if_block = ctx[5].cheatsheet && create_if_block_1(ctx);
  return {
    c() {
      if (if_block)
        if_block.c();
      if_block_anchor = empty();
    },
    m(target, anchor) {
      if (if_block)
        if_block.m(target, anchor);
      insert(target, if_block_anchor, anchor);
    },
    p(ctx2, dirty) {
      if (ctx2[5].cheatsheet) {
        if (if_block) {
          if_block.p(ctx2, dirty);
        } else {
          if_block = create_if_block_1(ctx2);
          if_block.c();
          if_block.m(if_block_anchor.parentNode, if_block_anchor);
        }
      } else if (if_block) {
        if_block.d(1);
        if_block = null;
      }
    },
    d(detaching) {
      if (if_block)
        if_block.d(detaching);
      if (detaching)
        detach(if_block_anchor);
    }
  };
}
function create_if_block$1(ctx) {
  let li;
  let div0;
  let raw_value = ctx[5].icon + "";
  let div1;
  let t0_value = ctx[5].title + "";
  let t0;
  let div2;
  let kbd;
  let t1_value = ctx[5].handler.shortcut + "";
  let t1;
  return {
    c() {
      li = element("li");
      div0 = element("div");
      div1 = element("div");
      t0 = text(t0_value);
      div2 = element("div");
      kbd = element("kbd");
      t1 = text(t1_value);
      attr(div0, "class", "bytemd-help-icon");
      attr(div1, "class", "bytemd-help-title");
      attr(div2, "class", "bytemd-help-content");
    },
    m(target, anchor) {
      insert(target, li, anchor);
      append(li, div0);
      div0.innerHTML = raw_value;
      append(li, div1);
      append(div1, t0);
      append(li, div2);
      append(div2, kbd);
      append(kbd, t1);
    },
    p(ctx2, dirty) {
      if (dirty & 4 && raw_value !== (raw_value = ctx2[5].icon + ""))
        div0.innerHTML = raw_value;
      if (dirty & 4 && t0_value !== (t0_value = ctx2[5].title + ""))
        set_data(t0, t0_value);
      if (dirty & 4 && t1_value !== (t1_value = ctx2[5].handler.shortcut + ""))
        set_data(t1, t1_value);
    },
    d(detaching) {
      if (detaching)
        detach(li);
    }
  };
}
function create_each_block(ctx) {
  let if_block_anchor;
  let if_block = ctx[5].handler && ctx[5].handler.type === "action" && ctx[5].handler.shortcut && create_if_block$1(ctx);
  return {
    c() {
      if (if_block)
        if_block.c();
      if_block_anchor = empty();
    },
    m(target, anchor) {
      if (if_block)
        if_block.m(target, anchor);
      insert(target, if_block_anchor, anchor);
    },
    p(ctx2, dirty) {
      if (ctx2[5].handler && ctx2[5].handler.type === "action" && ctx2[5].handler.shortcut) {
        if (if_block) {
          if_block.p(ctx2, dirty);
        } else {
          if_block = create_if_block$1(ctx2);
          if_block.c();
          if_block.m(if_block_anchor.parentNode, if_block_anchor);
        }
      } else if (if_block) {
        if_block.d(1);
        if_block = null;
      }
    },
    d(detaching) {
      if (if_block)
        if_block.d(detaching);
      if (detaching)
        detach(if_block_anchor);
    }
  };
}
function create_fragment$1(ctx) {
  let div;
  let h20;
  let t0_value = ctx[0].cheatsheet + "";
  let t0;
  let ul0;
  let h21;
  let t1_value = ctx[0].shortcuts + "";
  let t1;
  let ul1;
  let each_value_1 = ctx[2];
  let each_blocks_1 = [];
  for (let i = 0; i < each_value_1.length; i += 1) {
    each_blocks_1[i] = create_each_block_1(get_each_context_1(ctx, each_value_1, i));
  }
  let each_value = ctx[2];
  let each_blocks = [];
  for (let i = 0; i < each_value.length; i += 1) {
    each_blocks[i] = create_each_block(get_each_context(ctx, each_value, i));
  }
  return {
    c() {
      div = element("div");
      h20 = element("h2");
      t0 = text(t0_value);
      ul0 = element("ul");
      for (let i = 0; i < each_blocks_1.length; i += 1) {
        each_blocks_1[i].c();
      }
      h21 = element("h2");
      t1 = text(t1_value);
      ul1 = element("ul");
      for (let i = 0; i < each_blocks.length; i += 1) {
        each_blocks[i].c();
      }
      attr(div, "class", "bytemd-help");
      toggle_class(div, "bytemd-hidden", !ctx[1]);
    },
    m(target, anchor) {
      insert(target, div, anchor);
      append(div, h20);
      append(h20, t0);
      append(div, ul0);
      for (let i = 0; i < each_blocks_1.length; i += 1) {
        each_blocks_1[i].m(ul0, null);
      }
      append(div, h21);
      append(h21, t1);
      append(div, ul1);
      for (let i = 0; i < each_blocks.length; i += 1) {
        each_blocks[i].m(ul1, null);
      }
    },
    p(ctx2, [dirty]) {
      if (dirty & 1 && t0_value !== (t0_value = ctx2[0].cheatsheet + ""))
        set_data(t0, t0_value);
      if (dirty & 4) {
        each_value_1 = ctx2[2];
        let i;
        for (i = 0; i < each_value_1.length; i += 1) {
          const child_ctx = get_each_context_1(ctx2, each_value_1, i);
          if (each_blocks_1[i]) {
            each_blocks_1[i].p(child_ctx, dirty);
          } else {
            each_blocks_1[i] = create_each_block_1(child_ctx);
            each_blocks_1[i].c();
            each_blocks_1[i].m(ul0, null);
          }
        }
        for (; i < each_blocks_1.length; i += 1) {
          each_blocks_1[i].d(1);
        }
        each_blocks_1.length = each_value_1.length;
      }
      if (dirty & 1 && t1_value !== (t1_value = ctx2[0].shortcuts + ""))
        set_data(t1, t1_value);
      if (dirty & 4) {
        each_value = ctx2[2];
        let i;
        for (i = 0; i < each_value.length; i += 1) {
          const child_ctx = get_each_context(ctx2, each_value, i);
          if (each_blocks[i]) {
            each_blocks[i].p(child_ctx, dirty);
          } else {
            each_blocks[i] = create_each_block(child_ctx);
            each_blocks[i].c();
            each_blocks[i].m(ul1, null);
          }
        }
        for (; i < each_blocks.length; i += 1) {
          each_blocks[i].d(1);
        }
        each_blocks.length = each_value.length;
      }
      if (dirty & 2) {
        toggle_class(div, "bytemd-hidden", !ctx2[1]);
      }
    },
    i: noop,
    o: noop,
    d(detaching) {
      if (detaching)
        detach(div);
      destroy_each(each_blocks_1, detaching);
      destroy_each(each_blocks, detaching);
    }
  };
}
function instance$1($$self, $$props, $$invalidate) {
  let items;
  let { actions } = $$props;
  let { locale } = $$props;
  let { visible } = $$props;
  function flatItems(actions2) {
    let items2 = [];
    actions2.forEach((action) => {
      const { handler, cheatsheet: cheatsheet2 } = action;
      if ((handler == null ? void 0 : handler.type) === "dropdown") {
        items2.push(...flatItems(handler.actions));
      }
      if (cheatsheet2) {
        items2.push(action);
      }
    });
    return items2;
  }
  $$self.$$set = ($$props2) => {
    if ("actions" in $$props2)
      $$invalidate(3, actions = $$props2.actions);
    if ("locale" in $$props2)
      $$invalidate(0, locale = $$props2.locale);
    if ("visible" in $$props2)
      $$invalidate(1, visible = $$props2.visible);
  };
  $$self.$$.update = () => {
    if ($$self.$$.dirty & 8) {
      $$invalidate(2, items = flatItems(actions));
    }
  };
  return [locale, visible, items, actions];
}
class Help extends SvelteComponent {
  constructor(options) {
    super();
    init(this, options, instance$1, create_fragment$1, safe_not_equal, { actions: 3, locale: 0, visible: 1 });
  }
}
const bold = "Bold";
const boldText = "bold text";
const cheatsheet = "Markdown Cheatsheet";
const closeHelp = "Close help";
const closeToc = "Close table of contents";
const code = "Code";
const codeBlock = "Code block";
const codeLang = "lang";
const codeText = "code";
const exitFullscreen = "Exit fullscreen";
const exitPreviewOnly = "Exit preview only";
const exitWriteOnly = "Exit write only";
const fullscreen = "Fullscreen";
const h1 = "Heading 1";
const h2 = "Heading 2";
const h3 = "Heading 3";
const h4 = "Heading 4";
const h5 = "Heading 5";
const h6 = "Heading 6";
const headingText = "heading";
const help = "Help";
const hr = "Horizontal rule";
const image = "Image";
const imageAlt = "alt";
const imageTitle = "title";
const italic = "Italic";
const italicText = "italic text";
const limited = "The maximum character limit has been reached";
const lines = "Lines";
const link = "Link";
const linkText = "link text";
const ol = "Ordered list";
const olItem = "item";
const preview = "Preview";
const previewOnly = "Preview only";
const quote = "Quote";
const quotedText = "quoted text";
const shortcuts = "Shortcuts";
const source = "Source code";
const sync = "Scroll sync";
const toc = "Table of contents";
const top = "Scroll to top";
const ul = "Unordered list";
const ulItem = "item";
const words = "Words";
const write = "Write";
const writeOnly = "Write only";
var en = {
  bold,
  boldText,
  cheatsheet,
  closeHelp,
  closeToc,
  code,
  codeBlock,
  codeLang,
  codeText,
  exitFullscreen,
  exitPreviewOnly,
  exitWriteOnly,
  fullscreen,
  h1,
  h2,
  h3,
  h4,
  h5,
  h6,
  headingText,
  help,
  hr,
  image,
  imageAlt,
  imageTitle,
  italic,
  italicText,
  limited,
  lines,
  link,
  linkText,
  ol,
  olItem,
  preview,
  previewOnly,
  quote,
  quotedText,
  shortcuts,
  source,
  sync,
  toc,
  top,
  ul,
  ulItem,
  words,
  write,
  writeOnly
};
function create_if_block(ctx) {
  let viewer;
  let current;
  viewer = new Viewer({
    props: {
      value: ctx[16],
      plugins: ctx[1],
      sanitize: ctx[2],
      remarkRehype: ctx[3]
    }
  });
  viewer.$on("hast", ctx[35]);
  return {
    c() {
      create_component(viewer.$$.fragment);
    },
    m(target, anchor) {
      mount_component(viewer, target, anchor);
      current = true;
    },
    p(ctx2, dirty) {
      const viewer_changes = {};
      if (dirty[0] & 65536)
        viewer_changes.value = ctx2[16];
      if (dirty[0] & 2)
        viewer_changes.plugins = ctx2[1];
      if (dirty[0] & 4)
        viewer_changes.sanitize = ctx2[2];
      if (dirty[0] & 8)
        viewer_changes.remarkRehype = ctx2[3];
      viewer.$set(viewer_changes);
    },
    i(local) {
      if (current)
        return;
      transition_in(viewer.$$.fragment, local);
      current = true;
    },
    o(local) {
      transition_out(viewer.$$.fragment, local);
      current = false;
    },
    d(detaching) {
      destroy_component(viewer, detaching);
    }
  };
}
function create_fragment(ctx) {
  let div5;
  let toolbar;
  let div4;
  let div0;
  let div0_style_value;
  let div1;
  let div1_style_value;
  let div3;
  let div2;
  let raw_value = icons.Close({}) + "";
  let help2;
  let toc2;
  let status;
  let current;
  let mounted;
  let dispose;
  toolbar = new Toolbar({
    props: {
      context: ctx[10],
      split: ctx[11],
      activeTab: ctx[8],
      sidebar: ctx[9],
      fullscreen: ctx[15],
      locale: ctx[12],
      actions: ctx[21]
    }
  });
  toolbar.$on("key", ctx[31]);
  toolbar.$on("tab", ctx[32]);
  toolbar.$on("click", ctx[33]);
  let if_block = !ctx[4] && create_if_block(ctx);
  help2 = new Help({
    props: {
      locale: ctx[12],
      actions: ctx[21],
      visible: ctx[9] === "help"
    }
  });
  toc2 = new Toc({
    props: {
      hast: ctx[18],
      locale: ctx[12],
      currentBlockIndex: ctx[20],
      visible: ctx[9] === "toc"
    }
  });
  toc2.$on("click", ctx[38]);
  status = new Status({
    props: {
      locale: ctx[12],
      showSync: !ctx[4] && ctx[11],
      value: ctx[16],
      syncEnabled: ctx[17],
      islimited: ctx[0].length > ctx[5]
    }
  });
  status.$on("sync", ctx[39]);
  status.$on("top", ctx[40]);
  return {
    c() {
      div5 = element("div");
      create_component(toolbar.$$.fragment);
      div4 = element("div");
      div0 = element("div");
      div1 = element("div");
      if (if_block)
        if_block.c();
      div3 = element("div");
      div2 = element("div");
      create_component(help2.$$.fragment);
      create_component(toc2.$$.fragment);
      create_component(status.$$.fragment);
      attr(div0, "class", "bytemd-editor");
      attr(div0, "style", div0_style_value = ctx[22].edit);
      attr(div1, "class", "bytemd-preview");
      attr(div1, "style", div1_style_value = ctx[22].preview);
      attr(div2, "class", "bytemd-sidebar-close");
      attr(div3, "class", "bytemd-sidebar");
      toggle_class(div3, "bytemd-hidden", ctx[9] === false);
      attr(div4, "class", "bytemd-body");
      attr(div5, "class", "bytemd");
      toggle_class(div5, "bytemd-split", ctx[11] && ctx[8] === false);
      toggle_class(div5, "bytemd-fullscreen", ctx[15]);
    },
    m(target, anchor) {
      insert(target, div5, anchor);
      mount_component(toolbar, div5, null);
      append(div5, div4);
      append(div4, div0);
      ctx[34](div0);
      append(div4, div1);
      if (if_block)
        if_block.m(div1, null);
      ctx[36](div1);
      append(div4, div3);
      append(div3, div2);
      div2.innerHTML = raw_value;
      mount_component(help2, div3, null);
      mount_component(toc2, div3, null);
      mount_component(status, div5, null);
      ctx[41](div5);
      current = true;
      if (!mounted) {
        dispose = listen(div2, "click", ctx[37]);
        mounted = true;
      }
    },
    p(ctx2, dirty) {
      const toolbar_changes = {};
      if (dirty[0] & 1024)
        toolbar_changes.context = ctx2[10];
      if (dirty[0] & 2048)
        toolbar_changes.split = ctx2[11];
      if (dirty[0] & 256)
        toolbar_changes.activeTab = ctx2[8];
      if (dirty[0] & 512)
        toolbar_changes.sidebar = ctx2[9];
      if (dirty[0] & 32768)
        toolbar_changes.fullscreen = ctx2[15];
      if (dirty[0] & 4096)
        toolbar_changes.locale = ctx2[12];
      if (dirty[0] & 2097152)
        toolbar_changes.actions = ctx2[21];
      toolbar.$set(toolbar_changes);
      if (!current || dirty[0] & 4194304 && div0_style_value !== (div0_style_value = ctx2[22].edit)) {
        attr(div0, "style", div0_style_value);
      }
      if (!ctx2[4]) {
        if (if_block) {
          if_block.p(ctx2, dirty);
          if (dirty[0] & 16) {
            transition_in(if_block, 1);
          }
        } else {
          if_block = create_if_block(ctx2);
          if_block.c();
          transition_in(if_block, 1);
          if_block.m(div1, null);
        }
      } else if (if_block) {
        group_outros();
        transition_out(if_block, 1, 1, () => {
          if_block = null;
        });
        check_outros();
      }
      if (!current || dirty[0] & 4194304 && div1_style_value !== (div1_style_value = ctx2[22].preview)) {
        attr(div1, "style", div1_style_value);
      }
      const help_changes = {};
      if (dirty[0] & 4096)
        help_changes.locale = ctx2[12];
      if (dirty[0] & 2097152)
        help_changes.actions = ctx2[21];
      if (dirty[0] & 512)
        help_changes.visible = ctx2[9] === "help";
      help2.$set(help_changes);
      const toc_changes = {};
      if (dirty[0] & 262144)
        toc_changes.hast = ctx2[18];
      if (dirty[0] & 4096)
        toc_changes.locale = ctx2[12];
      if (dirty[0] & 1048576)
        toc_changes.currentBlockIndex = ctx2[20];
      if (dirty[0] & 512)
        toc_changes.visible = ctx2[9] === "toc";
      toc2.$set(toc_changes);
      if (dirty[0] & 512) {
        toggle_class(div3, "bytemd-hidden", ctx2[9] === false);
      }
      const status_changes = {};
      if (dirty[0] & 4096)
        status_changes.locale = ctx2[12];
      if (dirty[0] & 2064)
        status_changes.showSync = !ctx2[4] && ctx2[11];
      if (dirty[0] & 65536)
        status_changes.value = ctx2[16];
      if (dirty[0] & 131072)
        status_changes.syncEnabled = ctx2[17];
      if (dirty[0] & 33)
        status_changes.islimited = ctx2[0].length > ctx2[5];
      status.$set(status_changes);
      if (dirty[0] & 2304) {
        toggle_class(div5, "bytemd-split", ctx2[11] && ctx2[8] === false);
      }
      if (dirty[0] & 32768) {
        toggle_class(div5, "bytemd-fullscreen", ctx2[15]);
      }
    },
    i(local) {
      if (current)
        return;
      transition_in(toolbar.$$.fragment, local);
      transition_in(if_block);
      transition_in(help2.$$.fragment, local);
      transition_in(toc2.$$.fragment, local);
      transition_in(status.$$.fragment, local);
      current = true;
    },
    o(local) {
      transition_out(toolbar.$$.fragment, local);
      transition_out(if_block);
      transition_out(help2.$$.fragment, local);
      transition_out(toc2.$$.fragment, local);
      transition_out(status.$$.fragment, local);
      current = false;
    },
    d(detaching) {
      if (detaching)
        detach(div5);
      destroy_component(toolbar);
      ctx[34](null);
      if (if_block)
        if_block.d();
      ctx[36](null);
      destroy_component(help2);
      destroy_component(toc2);
      destroy_component(status);
      ctx[41](null);
      mounted = false;
      dispose();
    }
  };
}
function instance($$self, $$props, $$invalidate) {
  let mergedLocale;
  let actions;
  let split;
  let styles;
  let context;
  let { value = "" } = $$props;
  let { plugins = [] } = $$props;
  let { sanitize = void 0 } = $$props;
  let { remarkRehype: remarkRehype2 = void 0 } = $$props;
  let { mode = "auto" } = $$props;
  let { previewDebounce = 300 } = $$props;
  let { placeholder = void 0 } = $$props;
  let { editorConfig = void 0 } = $$props;
  let { locale = void 0 } = $$props;
  let { uploadImages = void 0 } = $$props;
  let { overridePreview = void 0 } = $$props;
  let { maxLength = Infinity } = $$props;
  const dispatch = createEventDispatcher();
  let root;
  let editorEl;
  let previewEl;
  let containerWidth = Infinity;
  let codemirror;
  let editor;
  let activeTab;
  let fullscreen2 = false;
  let sidebar = false;
  let cbs = [];
  let keyMap = {};
  function on() {
    cbs = plugins.map((p) => {
      var _a;
      return (_a = p.editorEffect) == null ? void 0 : _a.call(p, context);
    });
    keyMap = {};
    actions.forEach(({ handler }) => {
      if ((handler == null ? void 0 : handler.type) === "action" && handler.shortcut) {
        keyMap[handler.shortcut] = () => {
          handler.click(context);
        };
      }
    });
    editor.addKeyMap(keyMap);
  }
  function off() {
    cbs.forEach((cb) => cb && cb());
    editor == null ? void 0 : editor.removeKeyMap(keyMap);
  }
  let debouncedValue = value;
  const setDebouncedValue = debounce((value2) => {
    $$invalidate(16, debouncedValue = value2);
    overridePreview == null ? void 0 : overridePreview(previewEl, {
      value: debouncedValue,
      plugins,
      sanitize,
      remarkRehype: remarkRehype2
    });
  }, previewDebounce);
  let syncEnabled = true;
  let editCalled = false;
  let previewCalled = false;
  let editPs;
  let previewPs;
  let hast = { type: "root", children: [] };
  let vfile;
  let currentBlockIndex = 0;
  onMount(async () => {
    $$invalidate(30, codemirror = createCodeMirror());
    $$invalidate(7, editor = codemirror(editorEl, {
      value,
      mode: "yaml-frontmatter",
      lineWrapping: true,
      tabSize: 8,
      indentUnit: 4,
      extraKeys: {
        Enter: "newlineAndIndentContinueMarkdownList"
      },
      ...editorConfig,
      placeholder
    }));
    editor.addKeyMap({
      Tab: "indentMore",
      "Shift-Tab": "indentLess"
    });
    editor.on("change", () => {
      dispatch("change", { value: editor.getValue() });
    });
    const updateBlockPositions = throttle(() => {
      editPs = [];
      previewPs = [];
      const scrollInfo = editor.getScrollInfo();
      const body = previewEl.childNodes[0];
      if (!(body instanceof HTMLElement))
        return;
      const leftNodes = hast.children.filter((v) => v.type === "element");
      const rightNodes = [...body.childNodes].filter((v) => v instanceof HTMLElement);
      for (let i = 0; i < leftNodes.length; i++) {
        const leftNode = leftNodes[i];
        const rightNode = rightNodes[i];
        if (!leftNode.position) {
          continue;
        }
        const left = editor.heightAtLine(leftNode.position.start.line - 1, "local") / (scrollInfo.height - scrollInfo.clientHeight);
        const right = (rightNode.offsetTop - body.offsetTop) / (previewEl.scrollHeight - previewEl.clientHeight);
        if (left >= 1 || right >= 1) {
          break;
        }
        editPs.push(left);
        previewPs.push(right);
      }
      editPs.push(1);
      previewPs.push(1);
    }, 1e3);
    const editorScrollHandler = () => {
      if (overridePreview)
        return;
      if (!syncEnabled)
        return;
      if (previewCalled) {
        previewCalled = false;
        return;
      }
      updateBlockPositions();
      const info = editor.getScrollInfo();
      const leftRatio = info.top / (info.height - info.clientHeight);
      const startIndex = findStartIndex(leftRatio, editPs);
      const rightRatio = (leftRatio - editPs[startIndex]) * (previewPs[startIndex + 1] - previewPs[startIndex]) / (editPs[startIndex + 1] - editPs[startIndex]) + previewPs[startIndex];
      previewEl.scrollTo(0, rightRatio * (previewEl.scrollHeight - previewEl.clientHeight));
      editCalled = true;
    };
    const previewScrollHandler = () => {
      if (overridePreview)
        return;
      updateBlockPositions();
      $$invalidate(20, currentBlockIndex = findStartIndex(previewEl.scrollTop / (previewEl.scrollHeight - previewEl.offsetHeight), previewPs));
      if (!syncEnabled)
        return;
      if (editCalled) {
        editCalled = false;
        return;
      }
      const rightRatio = previewEl.scrollTop / (previewEl.scrollHeight - previewEl.clientHeight);
      const startIndex = findStartIndex(rightRatio, previewPs);
      const leftRatio = (rightRatio - previewPs[startIndex]) * (editPs[startIndex + 1] - editPs[startIndex]) / (previewPs[startIndex + 1] - previewPs[startIndex]) + editPs[startIndex];
      if (isNaN(leftRatio)) {
        return;
      }
      const info = editor.getScrollInfo();
      editor.scrollTo(0, leftRatio * (info.height - info.clientHeight));
      previewCalled = true;
    };
    editor.on("scroll", editorScrollHandler);
    previewEl.addEventListener("scroll", previewScrollHandler, { passive: true });
    const handleImages = async (e, itemList) => {
      if (!uploadImages)
        return;
      const files = Array.from(itemList != null ? itemList : []).map((item) => {
        if (item.type.startsWith("image/")) {
          return item.getAsFile();
        }
      }).filter((f) => f != null);
      if (files.length) {
        e.preventDefault();
        await handleImageUpload(context, uploadImages, files);
      }
    };
    editor.on("drop", async (_, e) => {
      var _a;
      handleImages(e, (_a = e.dataTransfer) == null ? void 0 : _a.items);
    });
    editor.on("paste", async (_, e) => {
      var _a;
      handleImages(e, (_a = e.clipboardData) == null ? void 0 : _a.items);
    });
    new ResizeObserver((entries) => {
      $$invalidate(29, containerWidth = entries[0].contentRect.width);
    }).observe(root, {
      box: "border-box"
    });
  });
  onDestroy(off);
  const key_handler = (e) => {
    editor.setOption("keyMap", e.detail);
    editor.focus();
  };
  const tab_handler = (e) => {
    const v = e.detail;
    if (split) {
      $$invalidate(8, activeTab = activeTab === v ? false : v);
    } else {
      $$invalidate(8, activeTab = v);
    }
    if (activeTab === "write") {
      tick().then(() => {
        editor && editor.focus();
      });
    }
  };
  const click_handler = (e) => {
    switch (e.detail) {
      case "fullscreen":
        $$invalidate(15, fullscreen2 = !fullscreen2);
        break;
      case "help":
        $$invalidate(9, sidebar = sidebar === "help" ? false : "help");
        break;
      case "toc":
        $$invalidate(9, sidebar = sidebar === "toc" ? false : "toc");
        break;
    }
  };
  function div0_binding($$value) {
    binding_callbacks[$$value ? "unshift" : "push"](() => {
      editorEl = $$value;
      $$invalidate(13, editorEl);
    });
  }
  const hast_handler = (e) => {
    $$invalidate(18, hast = e.detail.hast);
    $$invalidate(19, vfile = e.detail.file);
  };
  function div1_binding($$value) {
    binding_callbacks[$$value ? "unshift" : "push"](() => {
      previewEl = $$value;
      $$invalidate(14, previewEl);
    });
  }
  const click_handler_1 = () => {
    $$invalidate(9, sidebar = false);
  };
  const click_handler_2 = (e) => {
    const headings = previewEl.querySelectorAll("h1,h2,h3,h4,h5,h6");
    headings[e.detail].scrollIntoView();
  };
  const sync_handler = (e) => {
    $$invalidate(17, syncEnabled = e.detail);
  };
  const top_handler = () => {
    editor.scrollTo(null, 0);
    previewEl.scrollTo({ top: 0 });
  };
  function div5_binding($$value) {
    binding_callbacks[$$value ? "unshift" : "push"](() => {
      root = $$value;
      $$invalidate(6, root);
    });
  }
  $$self.$$set = ($$props2) => {
    if ("value" in $$props2)
      $$invalidate(0, value = $$props2.value);
    if ("plugins" in $$props2)
      $$invalidate(1, plugins = $$props2.plugins);
    if ("sanitize" in $$props2)
      $$invalidate(2, sanitize = $$props2.sanitize);
    if ("remarkRehype" in $$props2)
      $$invalidate(3, remarkRehype2 = $$props2.remarkRehype);
    if ("mode" in $$props2)
      $$invalidate(23, mode = $$props2.mode);
    if ("previewDebounce" in $$props2)
      $$invalidate(24, previewDebounce = $$props2.previewDebounce);
    if ("placeholder" in $$props2)
      $$invalidate(25, placeholder = $$props2.placeholder);
    if ("editorConfig" in $$props2)
      $$invalidate(26, editorConfig = $$props2.editorConfig);
    if ("locale" in $$props2)
      $$invalidate(27, locale = $$props2.locale);
    if ("uploadImages" in $$props2)
      $$invalidate(28, uploadImages = $$props2.uploadImages);
    if ("overridePreview" in $$props2)
      $$invalidate(4, overridePreview = $$props2.overridePreview);
    if ("maxLength" in $$props2)
      $$invalidate(5, maxLength = $$props2.maxLength);
  };
  $$self.$$.update = () => {
    if ($$self.$$.dirty[0] & 134217728) {
      $$invalidate(12, mergedLocale = { ...en, ...locale });
    }
    if ($$self.$$.dirty[0] & 268439554) {
      $$invalidate(21, actions = getBuiltinActions(mergedLocale, plugins, uploadImages));
    }
    if ($$self.$$.dirty[0] & 545259520) {
      $$invalidate(11, split = mode === "split" || mode === "auto" && containerWidth >= 800);
    }
    if ($$self.$$.dirty[0] & 2048) {
      ((_) => {
        if (split)
          $$invalidate(8, activeTab = false);
      })();
    }
    if ($$self.$$.dirty[0] & 2816) {
      $$invalidate(22, styles = (() => {
        let edit;
        let preview2;
        if (split && activeTab === false) {
          if (sidebar) {
            edit = `width:calc(50% - ${sidebar ? 140 : 0}px)`;
            preview2 = `width:calc(50% - ${sidebar ? 140 : 0}px)`;
          } else {
            edit = "width:50%";
            preview2 = "width:50%";
          }
        } else if (activeTab === "preview") {
          edit = "display:none";
          preview2 = `width:calc(100% - ${sidebar ? 280 : 0}px)`;
        } else {
          edit = `width:calc(100% - ${sidebar ? 280 : 0}px)`;
          preview2 = "display:none";
        }
        return { edit, preview: preview2 };
      })());
    }
    if ($$self.$$.dirty[0] & 1073742016) {
      $$invalidate(10, context = (() => {
        const context2 = {
          codemirror,
          editor,
          root,
          ...createEditorUtils(codemirror, editor)
        };
        return context2;
      })());
    }
    if ($$self.$$.dirty[0] & 1) {
      setDebouncedValue(value);
    }
    if ($$self.$$.dirty[0] & 129) {
      if (editor && value !== editor.getValue()) {
        editor.setValue(value);
      }
    }
    if ($$self.$$.dirty[0] & 130) {
      if (editor && plugins) {
        off();
        tick().then(() => {
          on();
        });
      }
    }
  };
  return [
    value,
    plugins,
    sanitize,
    remarkRehype2,
    overridePreview,
    maxLength,
    root,
    editor,
    activeTab,
    sidebar,
    context,
    split,
    mergedLocale,
    editorEl,
    previewEl,
    fullscreen2,
    debouncedValue,
    syncEnabled,
    hast,
    vfile,
    currentBlockIndex,
    actions,
    styles,
    mode,
    previewDebounce,
    placeholder,
    editorConfig,
    locale,
    uploadImages,
    containerWidth,
    codemirror,
    key_handler,
    tab_handler,
    click_handler,
    div0_binding,
    hast_handler,
    div1_binding,
    click_handler_1,
    click_handler_2,
    sync_handler,
    top_handler,
    div5_binding
  ];
}
class Editor extends SvelteComponent {
  constructor(options) {
    super();
    init(this, options, instance, create_fragment, not_equal, {
      value: 0,
      plugins: 1,
      sanitize: 2,
      remarkRehype: 3,
      mode: 23,
      previewDebounce: 24,
      placeholder: 25,
      editorConfig: 26,
      locale: 27,
      uploadImages: 28,
      overridePreview: 4,
      maxLength: 5
    }, null, [-1, -1]);
  }
}
export { Editor, Viewer, getProcessor };
